@@ -13,7 +13,7 @@
 #include <time.h>
 
 #include "str.h"
-#include "aux.h"
+#include "helpers.h"
 #include "crypto.h"
 #include "log.h"
 #include "call.h"
@@ -401,13 +401,13 @@ int dtls_init() {
 	return 0;
 }
 
-static void __dtls_timer(void *p) {
+static enum thread_looper_action __dtls_timer(void) {
 	struct dtls_cert *c;
 	long int left;
 
 	c = dtls_cert();
 	if (!c)
-		return;
+		return TLA_BREAK;
 
 	left = c->expires - rtpe_now.tv_sec;
 	if (left > CERT_EXPIRY_TIME/2)
@@ -417,10 +417,13 @@ static void __dtls_timer(void *p) {
 
 out:
 	obj_put(c);
+	return TLA_CONTINUE;
 }
 
-void dtls_timer(struct poller *p) {
-	poller_add_timer(p, __dtls_timer, NULL);
+void dtls_timer(void) {
+	thread_create_looper(__dtls_timer, rtpe_config.idle_scheduling,
+			rtpe_config.idle_priority, "DTLS refresh",
+			((long long) CERT_EXPIRY_TIME / 7) * 1000000);
 }
 
 static unsigned int generic_func(unsigned char *o, X509 *x, const EVP_MD *md) {
@@ -501,7 +504,13 @@ static int verify_callback(int ok, X509_STORE_CTX *store) {
 
 	if (ps->dtls_cert)
 		X509_free(ps->dtls_cert);
-	ps->dtls_cert = X509_dup(X509_STORE_CTX_get_current_cert(store));
+	ps->dtls_cert = NULL;
+	X509 *cert = X509_STORE_CTX_get0_cert(store);
+	if (!cert)
+		cert = X509_STORE_CTX_get_current_cert(store);
+	if (!cert)
+		return 0;
+	ps->dtls_cert = X509_dup(cert);
 
 	if (!media->fingerprint.hash_func || !media->fingerprint.digest_len)
 		return 1; /* delay verification */
@@ -670,6 +679,8 @@ int dtls_connection_init(struct dtls_connection *d, struct packet_stream *ps, in
 
 	d->active = active ? -1 : 0;
 
+	random_string(d->tls_id, sizeof(d->tls_id));
+
 done:
 	return 0;
 
@@ -862,13 +873,15 @@ int dtls(struct stream_fd *sfd, const str *s, const endpoint_t *fsin) {
 
 		if (!fsin) {
 			fsin = &ps->endpoint;
-			if (fsin->port == 9)
+			if (fsin->port == 9 || fsin->address.family == NULL)
 				fsin = NULL;
 		}
 
 		if (fsin) {
 			ilogs(srtp, LOG_DEBUG, "Sending DTLS packet");
 			socket_sendto(&sfd->socket, buf, ret, fsin);
+			atomic64_inc(&ps->stats_out.packets);
+			atomic64_add(&ps->stats_out.bytes, ret);
 		}
 	}
 
@@ -877,7 +890,6 @@ int dtls(struct stream_fd *sfd, const str *s, const endpoint_t *fsin) {
 
 /* call must be locked */
 void dtls_shutdown(struct packet_stream *ps) {
-
 	if (!ps)
 		return;
 
@@ -906,18 +918,15 @@ void dtls_shutdown(struct packet_stream *ps) {
 		}
 
 		dtls_connection_cleanup(d);
-
-		crypto_reset(&sfd->crypto);
 	}
 
-
 	if (ps->dtls_cert) {
 		X509_free(ps->dtls_cert);
 		ps->dtls_cert = NULL;
 	}
 
 	if (had_dtls)
-		call_stream_crypto_reset(ps);
+		ilogs(crypto, LOG_DEBUG, "Reuse SRTP crypto key");
 }
 
 void dtls_connection_cleanup(struct dtls_connection *c) {
