@@ -4,13 +4,14 @@
 #include <unistd.h>
 #include "str.h"
 #include "call.h"
-#include "aux.h"
+#include "helpers.h"
 #include "log.h"
 #include "obj.h"
 #include "stun.h"
 #include "poller.h"
 #include "log_funcs.h"
 #include "timerthread.h"
+#include "call_interfaces.h"
 
 
 
@@ -38,6 +39,19 @@
 
 
 
+struct fragment_key {
+	str call_id;
+	str from_tag;
+};
+struct sdp_fragment {
+	struct ng_buffer *ngbuf;
+	struct timeval received;
+	GQueue streams;
+	struct sdp_ng_flags flags;
+};
+
+
+
 static void __ice_agent_free(void *p);
 static void create_random_ice_string(struct call *call, str *s, int len);
 static void __do_ice_checks(struct ice_agent *ag);
@@ -79,6 +93,166 @@ const char * const ice_type_strings[] = {
 
 
 
+static mutex_t sdp_fragments_lock;
+static GHashTable *sdp_fragments;
+
+
+
+static void ice_update_media_streams(struct call_monologue *ml, GQueue *streams) {
+	for (GList *l = streams->head; l; l = l->next) {
+		struct stream_params *sp = l->data;
+		struct call_media *media = NULL;
+
+		if (sp->media_id.len)
+			media = g_hash_table_lookup(ml->media_ids, &sp->media_id);
+		else if (sp->index > 0) {
+			unsigned int arr_idx = sp->index - 1;
+			if (arr_idx < ml->medias->len)
+				media = ml->medias->pdata[arr_idx];
+		}
+
+		if (!media) {
+			ilogs(ice, LOG_WARN, "No matching media for trickle ICE update found");
+			continue;
+		}
+
+		if (!media->ice_agent) {
+			ilogs(ice, LOG_WARN, "Media for trickle ICE update is not ICE-enabled");
+			continue;
+		}
+		if (!MEDIA_ISSET(media, TRICKLE_ICE)) {
+			ilogs(ice, LOG_WARN, "Media for trickle ICE update is not trickle-ICE-enabled");
+			continue;
+		}
+
+		ice_update(media->ice_agent, sp, false);
+	}
+}
+
+
+static unsigned int frag_key_hash(const void *A) {
+	const struct fragment_key *a = A;
+	return str_hash(&a->call_id) ^ str_hash(&a->from_tag);
+}
+static int frag_key_eq(const void *A, const void *B) {
+	const struct fragment_key *a = A;
+	const struct fragment_key *b = B;
+	return str_equal(&a->call_id, &b->call_id)
+		&& str_equal(&a->from_tag, &b->from_tag);
+}
+
+static void fragment_free(struct sdp_fragment *frag) {
+	sdp_streams_free(&frag->streams);
+	call_ng_free_flags(&frag->flags);
+	obj_put(frag->ngbuf);
+	g_slice_free1(sizeof(*frag), frag);
+}
+static void fragment_key_free(void *p) {
+	struct fragment_key *k = p;
+	g_free(k->call_id.s);
+	g_free(k->from_tag.s);
+	g_slice_free1(sizeof(*k), k);
+}
+static void queue_sdp_fragment(struct ng_buffer *ngbuf, GQueue *streams, struct sdp_ng_flags *flags) {
+	ilog(LOG_DEBUG, "Queuing up SDP fragment for " STR_FORMAT_M "/" STR_FORMAT_M,
+			STR_FMT_M(&flags->call_id), STR_FMT_M(&flags->from_tag));
+
+	struct fragment_key *k = g_slice_alloc0(sizeof(*k));
+	str_init_dup_str(&k->call_id, &flags->call_id);
+	str_init_dup_str(&k->from_tag, &flags->from_tag);
+
+	struct sdp_fragment *frag = g_slice_alloc0(sizeof(*frag));
+	frag->received = rtpe_now;
+	frag->ngbuf = obj_get(ngbuf);
+	frag->streams = *streams;
+	frag->flags = *flags;
+	g_queue_init(streams);
+	ZERO(*flags);
+
+	mutex_lock(&sdp_fragments_lock);
+	GQueue *frags = g_hash_table_lookup_queue_new(sdp_fragments, k, fragment_key_free);
+	g_queue_push_tail(frags, frag);
+	mutex_unlock(&sdp_fragments_lock);
+}
+bool trickle_ice_update(struct ng_buffer *ngbuf, struct call *call, struct sdp_ng_flags *flags,
+		GQueue *streams)
+{
+	if (!flags->fragment)
+		return false;
+
+	if (!call) {
+		queue_sdp_fragment(ngbuf, streams, flags);
+		return true;
+	}
+	struct call_monologue *ml = call_get_monologue(call, &flags->from_tag);
+	if (!ml) {
+		queue_sdp_fragment(ngbuf, streams, flags);
+		return true;
+	}
+
+	ice_update_media_streams(ml, streams);
+
+	return true;
+}
+#define MAX_FRAG_AGE 3000000
+void dequeue_sdp_fragments(struct call_monologue *monologue) {
+	struct fragment_key k;
+	ZERO(k);
+	k.call_id = monologue->call->callid;
+	k.from_tag = monologue->tag;
+
+	GQueue *frags = NULL;
+
+	{
+		LOCK(&sdp_fragments_lock);
+		g_hash_table_steal_extended(sdp_fragments, &k, NULL, (void **) &frags);
+		if (!frags)
+			return;
+
+		// we own the queue now
+	}
+
+	struct sdp_fragment *frag;
+	while ((frag = g_queue_pop_head(frags))) {
+		if (timeval_diff(&rtpe_now, &frag->received) > MAX_FRAG_AGE)
+			goto next;
+
+		ilog(LOG_DEBUG, "Dequeuing SDP fragment for " STR_FORMAT_M "/" STR_FORMAT_M,
+				STR_FMT_M(&k.call_id), STR_FMT_M(&k.from_tag));
+
+		ice_update_media_streams(monologue, &frag->streams);
+
+next:
+		fragment_free(frag);
+	}
+
+	g_queue_free(frags);
+}
+static gboolean fragment_check_cleanup(void *k, void *v, void *p) {
+	bool all = GPOINTER_TO_INT(p);
+	struct fragment_key *key = k;
+	GQueue *frags = v;
+	if (!key || !frags)
+		return TRUE;
+	while (frags->length) {
+		struct sdp_fragment *frag = frags->head->data;
+		if (!all && timeval_diff(&rtpe_now, &frag->received) <= MAX_FRAG_AGE)
+			break;
+		g_queue_pop_head(frags);
+		fragment_free(frag);
+	}
+	if (!frags->length) {
+		g_queue_free(frags);
+		return TRUE;
+	}
+	return FALSE;
+}
+static void fragments_cleanup(bool all) {
+	mutex_lock(&sdp_fragments_lock);
+	g_hash_table_foreach_remove(sdp_fragments, fragment_check_cleanup, GINT_TO_POINTER(all));
+	mutex_unlock(&sdp_fragments_lock);
+}
+
 
 
 enum ice_candidate_type ice_candidate_type(const str *s) {
@@ -329,6 +503,8 @@ void ice_update(struct ice_agent *ag, struct stream_params *sp, bool allow_reset
 	if (!ag)
 		return;
 
+	log_info_ice_agent(ag);
+
 	atomic64_set(&ag->last_activity, rtpe_now.tv_sec);
 	media = ag->media;
 	call = media->call;
@@ -359,7 +535,7 @@ void ice_update(struct ice_agent *ag, struct stream_params *sp, bool allow_reset
 	comps = 0;
 	for (l = media->streams.head; l; l = l->next)
 		components[comps++] = l->data;
-	if (comps == 2 && MEDIA_ISSET(media, RTCP_MUX))
+	if (comps == 2 && (MEDIA_ISSET(media, RTCP_MUX) || !proto_is_rtp(media->protocol)))
 		components[1] = NULL;
 
 	comps = 0;
@@ -460,6 +636,8 @@ pair:
 		__do_ice_checks(ag);
 	else
 		__agent_shutdown(ag);
+
+	log_info_pop();
 }
 
 
@@ -566,13 +744,24 @@ static void __agent_deschedule(struct ice_agent *ag) {
 void ice_init(void) {
 	random_string((void *) &tie_breaker, sizeof(tie_breaker));
 	timerthread_init(&ice_agents_timer_thread, ice_agents_timer_run);
+
+	sdp_fragments = g_hash_table_new_full(frag_key_hash, frag_key_eq, fragment_key_free, NULL);
+	mutex_init(&sdp_fragments_lock);
 }
 
 void ice_free(void) {
 	timerthread_free(&ice_agents_timer_thread);
+
+	fragments_cleanup(true);
+	g_hash_table_destroy(sdp_fragments);
+	sdp_fragments = NULL;
+	mutex_destroy(&sdp_fragments_lock);
 }
 
-
+enum thread_looper_action ice_slow_timer(void) {
+	fragments_cleanup(false);
+	return TLA_CONTINUE;
+}
 
 static void __fail_pair(struct ice_candidate_pair *pair) {
 	ilogs(ice, LOG_DEBUG, "Setting ICE candidate pair "PAIR_FORMAT" as failed", PAIR_FMT(pair));
@@ -1081,7 +1270,7 @@ static int __check_valid(struct ice_agent *ag) {
 		}
 	}
 
-	call_media_unkernelize(media);
+	call_media_unkernelize(media, "ICE negotiation event");
 
 	g_queue_clear(&all_compos);
 	return 1;
@@ -1396,3 +1585,20 @@ void ice_remote_candidates(GQueue *out, struct ice_agent *ag) {
 
 	g_queue_clear(&all_compos);
 }
+
+bool ice_peer_address_known(struct ice_agent *ag, const endpoint_t *sin, struct packet_stream *ps,
+		const struct local_intf *ifa)
+{
+	LOCK(&ag->lock);
+
+	struct ice_candidate *cand = __cand_lookup(ag, sin, ps->component);
+	if (!cand)
+		return false;
+	struct ice_candidate_pair *pair = __pair_lookup(ag, cand, ifa);
+	if (!pair)
+		return false;
+	if (!PAIR_ISSET(pair, VALID))
+		return false;
+
+	return true;
+}
