@@ -21,6 +21,7 @@
 #include "rtplib.h"
 #include "cdr.h"
 #include "log.h"
+#include "call_interfaces.h"
 
 
 
@@ -39,12 +40,20 @@ static int append_meta_chunk(struct recording *recording, const char *buf, unsig
 		const char *label_fmt, ...)
 	__attribute__((format(printf,4,5)));
 
+// all methods
+static int create_spool_dir_all(const char *spoolpath);
+static void init_all(struct call *call);
+static void sdp_after_all(struct recording *recording, GString *str, struct call_monologue *ml,
+		enum call_opmode opmode);
+static void dump_packet_all(struct media_packet *mp, const str *s);
+static void finish_all(struct call *call, bool discard);
+
 // pcap methods
 static int rec_pcap_create_spool_dir(const char *dirpath);
 static void rec_pcap_init(struct call *);
 static void sdp_after_pcap(struct recording *, GString *str, struct call_monologue *, enum call_opmode opmode);
 static void dump_packet_pcap(struct media_packet *mp, const str *s);
-static void finish_pcap(struct call *);
+static void finish_pcap(struct call *, bool discard);
 static void response_pcap(struct recording *, bencode_item_t *);
 
 // proc methods
@@ -53,7 +62,7 @@ static void sdp_before_proc(struct recording *, const str *, struct call_monolog
 static void sdp_after_proc(struct recording *, GString *str, struct call_monologue *, enum call_opmode opmode);
 static void meta_chunk_proc(struct recording *, const char *, const str *);
 static void update_flags_proc(struct call *call, bool streams);
-static void finish_proc(struct call *);
+static void finish_proc(struct call *, bool discard);
 static void dump_packet_proc(struct media_packet *mp, const str *s);
 static void init_stream_proc(struct packet_stream *);
 static void setup_stream_proc(struct packet_stream *);
@@ -68,15 +77,23 @@ static void rec_pcap_eth_header(unsigned char *, struct packet_stream *);
 #define append_meta_chunk_null(r,f...) append_meta_chunk(r, "", 0, f)
 
 
-static const struct recording_method methods[] = {
+const struct recording_method methods[] = {
 	{
 		.name = "pcap",
 		.kernel_support = 0,
 		.create_spool_dir = rec_pcap_create_spool_dir,
 		.init_struct = rec_pcap_init,
+		.sdp_before = NULL,
 		.sdp_after = sdp_after_pcap,
+		.meta_chunk = NULL,
+		.update_flags = NULL,
 		.dump_packet = dump_packet_pcap,
 		.finish = finish_pcap,
+		.init_stream_struct = NULL,
+		.setup_stream = NULL,
+		.setup_media = NULL,
+		.setup_monologue = NULL,
+		.stream_kernel_info = NULL,
 		.response = response_pcap,
 	},
 	{
@@ -95,6 +112,25 @@ static const struct recording_method methods[] = {
 		.setup_media = setup_media_proc,
 		.setup_monologue = setup_monologue_proc,
 		.stream_kernel_info = kernel_info_proc,
+		.response = NULL,
+	},
+	{
+		.name = "all",
+		.kernel_support = 0,
+		.create_spool_dir = create_spool_dir_all,
+		.init_struct = init_all,
+		.sdp_before = sdp_before_proc,
+		.sdp_after = sdp_after_all,
+		.meta_chunk = meta_chunk_proc,
+		.update_flags = update_flags_proc,
+		.dump_packet = dump_packet_all,
+		.finish = finish_all,
+		.init_stream_struct = init_stream_proc,
+		.setup_stream = setup_stream_proc,
+		.setup_media = setup_media_proc,
+		.setup_monologue = setup_monologue_proc,
+		.stream_kernel_info = kernel_info_proc,
+		.response = response_pcap,
 	},
 };
 
@@ -123,10 +159,7 @@ static const struct rec_pcap_format *rec_pcap_format;
  */
 
 void recording_fs_free(void) {
-	if (spooldir)
-		free(spooldir);
-
-	spooldir = NULL;
+	g_clear_pointer(&spooldir, free);
 }
 
 /**
@@ -262,7 +295,7 @@ void update_metadata_monologue(struct call_monologue *ml, str *metadata) {
 	update_metadata_call(call, metadata);
 }
 
-static void update_output_dest(struct call *call, str *output_dest) {
+static void update_output_dest(struct call *call, const str *output_dest) {
 	if (!output_dest || !output_dest->s || !call->recording)
 		return;
 	recording_meta_chunk(call->recording, "OUTPUT_DESTINATION", output_dest);
@@ -270,14 +303,14 @@ static void update_output_dest(struct call *call, str *output_dest) {
 
 // lock must be held
 static void update_flags_proc(struct call *call, bool streams) {
-	append_meta_chunk_null(call->recording, "RECORDING %u", call->recording_on ? 1 : 0);
-	append_meta_chunk_null(call->recording, "FORWARDING %u", call->rec_forwarding ? 1 : 0);
+	append_meta_chunk_null(call->recording, "RECORDING %u", CALL_ISSET(call, RECORDING_ON));
+	append_meta_chunk_null(call->recording, "FORWARDING %u", CALL_ISSET(call, REC_FORWARDING));
 	if (!streams)
 		return;
 	for (GList *l = call->streams.head; l; l = l->next) {
 		struct packet_stream *ps = l->data;
 		append_meta_chunk_null(call->recording, "STREAM %u FORWARDING %u",
-				ps->unique_id, ps->media->monologue->rec_forwarding ? 1 : 0);
+				ps->unique_id, ML_ISSET(ps->media->monologue, REC_FORWARDING) ? 1 : 0);
 	}
 }
 static void recording_update_flags(struct call *call, bool streams) {
@@ -285,11 +318,10 @@ static void recording_update_flags(struct call *call, bool streams) {
 }
 
 // lock must be held
-void recording_start(struct call *call, const char *prefix, str *output_dest) {
-	update_output_dest(call, output_dest);
-
+void recording_start(struct call *call, const char *prefix, const str *output_dest) {
 	if (call->recording) {
 		// already active
+		update_output_dest(call, output_dest);
 		recording_update_flags(call, true);
 		return;
 	}
@@ -307,17 +339,17 @@ void recording_start(struct call *call, const char *prefix, str *output_dest) {
 		const int rand_bytes = 8;
 		char rand_str[rand_bytes * 2 + 1];
 		rand_hex_str(rand_str, rand_bytes);
-		if (asprintf(&recording->meta_prefix, "%s-%s", recording->escaped_callid, rand_str) < 0)
-			abort();
+		recording->meta_prefix = g_strdup_printf("%s-%s", recording->escaped_callid, rand_str);
 	}
 	else
-		recording->meta_prefix = strdup(prefix);
+		recording->meta_prefix = g_strdup(prefix);
 
 	_rm(init_struct, call);
 
 	// update main call flags (global recording/forwarding on/off) to prevent recording
 	// features from being started when the stream info (through setup_stream) is
 	// propagated if recording is actually off
+	update_output_dest(call, output_dest);
 	recording_update_flags(call, false);
 
 	// if recording has been turned on after initial call setup, we must walk
@@ -336,7 +368,7 @@ void recording_start(struct call *call, const char *prefix, str *output_dest) {
 	for (l = call->streams.head; l; l = l->next) {
 		struct packet_stream *ps = l->data;
 		recording_setup_stream(ps);
-		__unkernelize(ps);
+		__unkernelize(ps, "recording start");
 		__reset_sink_handlers(ps);
 	}
 
@@ -347,22 +379,34 @@ void recording_stop(struct call *call) {
 		return;
 
 	// check if all recording options are disabled
-	if (call->recording_on || call->rec_forwarding) {
+	if (CALL_ISSET(call, RECORDING_ON) || CALL_ISSET(call, REC_FORWARDING)) {
 		recording_update_flags(call, true);
 		return;
 	}
 
 	for (GList *l = call->monologues.head; l; l = l->next) {
 		struct call_monologue *ml = l->data;
-		if (ml->rec_forwarding) {
+		if (ML_ISSET(ml, REC_FORWARDING)) {
 			recording_update_flags(call, true);
 			return;
 		}
 	}
 
 	ilog(LOG_NOTICE, "Turning off call recording.");
-	recording_finish(call);
+	recording_finish(call, false);
 }
+void recording_pause(struct call *call) {
+	ilog(LOG_NOTICE, "Pausing call recording.");
+	recording_update_flags(call, true);
+}
+void recording_discard(struct call *call) {
+	CALL_CLEAR(call, RECORDING_ON);
+	if (!call->recording)
+		return;
+	ilog(LOG_NOTICE, "Turning off call recording and discarding outputs.");
+	recording_finish(call, true);
+}
+
 
 /**
  *
@@ -374,19 +418,25 @@ void recording_stop(struct call *call) {
  *
  * Returns a boolean for whether or not the call is being recorded.
  */
-void detect_setup_recording(struct call *call, const str *recordcall) {
-	if (!recordcall || !recordcall->s)
+void detect_setup_recording(struct call *call, const struct sdp_ng_flags *flags) {
+	if (!flags)
 		return;
 
-	if (!str_cmp(recordcall, "yes") || !str_cmp(recordcall, "on")) {
-		call->recording_on = 1;
-		recording_start(call, NULL, NULL);
+	const str *recordcall = &flags->record_call_str;
+
+	if (!str_cmp(recordcall, "yes") || !str_cmp(recordcall, "on") || flags->record_call) {
+		CALL_SET(call, RECORDING_ON);
+		recording_start(call, NULL, &flags->output_dest);
 	}
 	else if (!str_cmp(recordcall, "no") || !str_cmp(recordcall, "off")) {
-		call->recording_on = 0;
+		CALL_CLEAR(call, RECORDING_ON);
 		recording_stop(call);
 	}
-	else
+	else if (!str_cmp(recordcall, "discard") || flags->discard_recording) {
+		CALL_CLEAR(call, RECORDING_ON);
+		recording_discard(call);
+	}
+	else if (recordcall->len != 0)
 		ilog(LOG_INFO, "\"record-call\" flag "STR_FORMAT" is invalid flag.", STR_FMT(recordcall));
 }
 
@@ -394,24 +444,21 @@ static void rec_pcap_init(struct call *call) {
 	struct recording *recording = call->recording;
 
 	// Wireshark starts at packet index 1, so we start there, too
-	recording->u.pcap.packet_num = 1;
-	mutex_init(&recording->u.pcap.recording_lock);
+	recording->pcap.packet_num = 1;
+	mutex_init(&recording->pcap.recording_lock);
 	meta_setup_file(recording);
 
 	// set up pcap file
 	char *pcap_path = recording_setup_file(recording);
-	if (pcap_path != NULL && recording->u.pcap.recording_pdumper != NULL
-	    && recording->u.pcap.meta_fp) {
+	if (pcap_path != NULL && recording->pcap.recording_pdumper != NULL
+	    && recording->pcap.meta_fp) {
 		// Write the location of the PCAP file to the metadata file
-		fprintf(recording->u.pcap.meta_fp, "%s\n\n", pcap_path);
+		fprintf(recording->pcap.meta_fp, "%s\n\n", pcap_path);
 	}
 }
 
 static char *file_path_str(const char *id, const char *prefix, const char *suffix) {
-	char *ret;
-	if (asprintf(&ret, "%s%s%s%s", spooldir, prefix, id, suffix) < 0)
-		abort();
-	return ret;
+	return g_strdup_printf("%s%s%s%s", spooldir, prefix, id, suffix);
 }
 
 /**
@@ -425,17 +472,16 @@ static char *meta_setup_file(struct recording *recording) {
 	}
 
 	char *meta_filepath = file_path_str(recording->meta_prefix, "/tmp/rtpengine-meta-", ".tmp");
-	recording->meta_filepath = meta_filepath;
+	recording->pcap.meta_filepath = meta_filepath;
 	FILE *mfp = fopen(meta_filepath, "w");
 	// coverity[check_return : FALSE]
 	chmod(meta_filepath, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH);
 	if (mfp == NULL) {
 		ilog(LOG_ERROR, "Could not open metadata file: %s%s%s", FMT_M(meta_filepath));
-		free(meta_filepath);
-		recording->meta_filepath = NULL;
+		g_clear_pointer(&recording->pcap.meta_filepath, free);
 		return NULL;
 	}
-	recording->u.pcap.meta_fp = mfp;
+	recording->pcap.meta_fp = mfp;
 	ilog(LOG_DEBUG, "Wrote metadata file to temporary path: %s%s%s", FMT_M(meta_filepath));
 	return meta_filepath;
 }
@@ -443,10 +489,10 @@ static char *meta_setup_file(struct recording *recording) {
 /**
  * Write out a block of SDP to the metadata file.
  */
-static void sdp_after_pcap(struct recording *recording, GString *str, struct call_monologue *ml,
+static void sdp_after_pcap(struct recording *recording, GString *s, struct call_monologue *ml,
 		enum call_opmode opmode)
 {
-	FILE *meta_fp = recording->u.pcap.meta_fp;
+	FILE *meta_fp = recording->pcap.meta_fp;
 	if (!meta_fp)
 		return;
 
@@ -461,24 +507,22 @@ static void sdp_after_pcap(struct recording *recording, GString *str, struct cal
 	fprintf(meta_fp, "%.3lf", ml->started.tv_sec*1000.0+ml->started.tv_usec/1000.0);
 	fprintf(meta_fp, "\nSDP mode: ");
 	fprintf(meta_fp, "%s", get_opmode_text(opmode));
-	fprintf(meta_fp, "\nSDP before RTP packet: %" PRIu64 "\n\n", recording->u.pcap.packet_num);
+	fprintf(meta_fp, "\nSDP before RTP packet: %" PRIu64 "\n\n", recording->pcap.packet_num);
 	fflush(meta_fp);
-	if (write(meta_fd, str->str, str->len) <= 0)
+	if (write(meta_fd, s->str, s->len) <= 0)
 		ilog(LOG_WARN, "Error writing SDP body to metadata file: %s", strerror(errno));
 }
 
 /**
  * Writes metadata to metafile, closes file, and renames it to finished location.
- * Returns non-zero for failure.
  */
-static int rec_pcap_meta_finish_file(struct call *call) {
+static void rec_pcap_meta_finish_file(struct call *call) {
 	// This should usually be called from a place that has the call->master_lock
 	struct recording *recording = call->recording;
-	int return_code = 0;
 
-	if (recording == NULL || recording->u.pcap.meta_fp == NULL) {
+	if (recording == NULL || recording->pcap.meta_fp == NULL) {
 		ilog(LOG_INFO, "Trying to clean up recording meta file without a file pointer opened.");
-		return 0;
+		return;
 	}
 
 	// Print start timestamp and end timestamp
@@ -489,34 +533,34 @@ static int rec_pcap_meta_finish_file(struct call *call) {
 	struct tm timeinfo;
 	struct timeval *terminate;
 	terminate = &(((struct call_monologue *)call->monologues.head->data)->terminated);
-	fprintf(recording->u.pcap.meta_fp, "\nTimestamp terminated ms(first monologue): %.3lf", terminate->tv_sec*1000.0 + terminate->tv_usec/1000.0);
+	fprintf(recording->pcap.meta_fp, "\nTimestamp terminated ms(first monologue): %.3lf", terminate->tv_sec*1000.0 + terminate->tv_usec/1000.0);
 	if (localtime_r(&start, &timeinfo) == NULL) {
 		ilog(LOG_ERROR, "Cannot get start local time, while cleaning up recording meta file: %s", strerror(errno));
 	} else {
 		strftime(timebuffer, 20, "%FT%T", &timeinfo);
-		fprintf(recording->u.pcap.meta_fp, "\n\ncall start time: %s\n", timebuffer);
+		fprintf(recording->pcap.meta_fp, "\n\ncall start time: %s\n", timebuffer);
 	}
 	if (localtime_r(&end, &timeinfo) == NULL) {
 		ilog(LOG_ERROR, "Cannot get end local time, while cleaning up recording meta file: %s", strerror(errno));
 	} else {
 		strftime(timebuffer, 20, "%FT%T", &timeinfo);
-		fprintf(recording->u.pcap.meta_fp, "call end time: %s\n", timebuffer);
+		fprintf(recording->pcap.meta_fp, "call end time: %s\n", timebuffer);
 	}
 
 	// Print metadata
 	if (call->metadata.len)
-		fprintf(recording->u.pcap.meta_fp, "\n\n"STR_FORMAT"\n", STR_FMT(&call->metadata));
-	fclose(recording->u.pcap.meta_fp);
-	recording->u.pcap.meta_fp = NULL;
+		fprintf(recording->pcap.meta_fp, "\n\n"STR_FORMAT"\n", STR_FMT(&call->metadata));
+	fclose(recording->pcap.meta_fp);
+	recording->pcap.meta_fp = NULL;
 
 	// Get the filename (in between its directory and the file extension)
 	// and move it to the finished file location.
 	// Rename extension to ".txt".
 	int fn_len;
-	char *meta_filename = strrchr(recording->meta_filepath, '/');
+	char *meta_filename = strrchr(recording->pcap.meta_filepath, '/');
 	char *meta_ext = NULL;
 	if (meta_filename == NULL) {
-		meta_filename = recording->meta_filepath;
+		meta_filename = recording->pcap.meta_filepath;
 	}
 	else {
 		meta_filename = meta_filename + 1;
@@ -529,18 +573,34 @@ static int rec_pcap_meta_finish_file(struct call *call) {
 	char new_metapath[prefix_len + fn_len + ext_len + 1];
 	snprintf(new_metapath, prefix_len+fn_len+1, "%s/metadata/%s", spooldir, meta_filename);
 	snprintf(new_metapath + prefix_len+fn_len, ext_len+1, ".txt");
-	return_code = return_code || rename(recording->meta_filepath, new_metapath);
+	int return_code = rename(recording->pcap.meta_filepath, new_metapath);
 	if (return_code != 0) {
 		ilog(LOG_ERROR, "Could not move metadata file \"%s\" to \"%s/metadata/\"",
-				 recording->meta_filepath, spooldir);
+				 recording->pcap.meta_filepath, spooldir);
 	} else {
 		ilog(LOG_INFO, "Moved metadata file \"%s\" to \"%s/metadata\"",
-				 recording->meta_filepath, spooldir);
+				 recording->pcap.meta_filepath, spooldir);
 	}
 
-	mutex_destroy(&recording->u.pcap.recording_lock);
+	mutex_destroy(&recording->pcap.recording_lock);
+	g_clear_pointer(&recording->pcap.meta_filepath, g_free);
+}
 
-	return return_code;
+/**
+ * Closes and discards all output files.
+ */
+static void rec_pcap_meta_discard_file(struct call *call) {
+	struct recording *recording = call->recording;
+
+	if (recording == NULL || recording->pcap.meta_fp == NULL)
+		return;
+
+	fclose(recording->pcap.meta_fp);
+	recording->pcap.meta_fp = NULL;
+
+	unlink(recording->pcap.recording_path);
+	unlink(recording->pcap.meta_filepath);
+	g_clear_pointer(&recording->pcap.meta_filepath, free);
 }
 
 /**
@@ -552,17 +612,17 @@ static char *recording_setup_file(struct recording *recording) {
 
 	if (!spooldir)
 		return NULL;
-	if (recording->u.pcap.recording_pd || recording->u.pcap.recording_pdumper)
+	if (recording->pcap.recording_pd || recording->pcap.recording_pdumper)
 		return NULL;
 
 	recording_path = file_path_str(recording->meta_prefix, "/pcaps/", ".pcap");
-	recording->u.pcap.recording_path = recording_path;
+	recording->pcap.recording_path = recording_path;
 
-	recording->u.pcap.recording_pd = pcap_open_dead(rec_pcap_format->linktype, 65535);
-	recording->u.pcap.recording_pdumper = pcap_dump_open(recording->u.pcap.recording_pd, recording_path);
-	if (recording->u.pcap.recording_pdumper == NULL) {
-		pcap_close(recording->u.pcap.recording_pd);
-		recording->u.pcap.recording_pd = NULL;
+	recording->pcap.recording_pd = pcap_open_dead(rec_pcap_format->linktype, 65535);
+	recording->pcap.recording_pdumper = pcap_dump_open(recording->pcap.recording_pd, recording_path);
+	if (recording->pcap.recording_pdumper == NULL) {
+		pcap_close(recording->pcap.recording_pd);
+		recording->pcap.recording_pd = NULL;
 		ilog(LOG_INFO, "Failed to write recording file: %s", recording_path);
 	} else {
 		ilog(LOG_INFO, "Writing recording file: %s", recording_path);
@@ -575,13 +635,13 @@ static char *recording_setup_file(struct recording *recording) {
  * Flushes PCAP file, closes the dumper and descriptors, and frees object memory.
  */
 static void rec_pcap_recording_finish_file(struct recording *recording) {
-	if (recording->u.pcap.recording_pdumper != NULL) {
-		pcap_dump_flush(recording->u.pcap.recording_pdumper);
-		pcap_dump_close(recording->u.pcap.recording_pdumper);
-		free(recording->u.pcap.recording_path);
+	if (recording->pcap.recording_pdumper != NULL) {
+		pcap_dump_flush(recording->pcap.recording_pdumper);
+		pcap_dump_close(recording->pcap.recording_pdumper);
+		g_clear_pointer(&recording->pcap.recording_path, free);
 	}
-	if (recording->u.pcap.recording_pd != NULL) {
-		pcap_close(recording->u.pcap.recording_pd);
+	if (recording->pcap.recording_pd != NULL) {
+		pcap_close(recording->pcap.recording_pd);
 	}
 }
 
@@ -612,7 +672,7 @@ static void rec_pcap_eth_header(unsigned char *pkt, struct packet_stream *stream
  * A fair amount extraneous of packet data is spoofed.
  */
 static void stream_pcap_dump(struct media_packet *mp, const str *s) {
-	pcap_dumper_t *pdumper = mp->call->recording->u.pcap.recording_pdumper;
+	pcap_dumper_t *pdumper = mp->call->recording->pcap.recording_pdumper;
 	if (!pdumper)
 		return;
 
@@ -634,24 +694,29 @@ static void stream_pcap_dump(struct media_packet *mp, const str *s) {
 }
 
 static void dump_packet_pcap(struct media_packet *mp, const str *s) {
+	if (ML_ISSET(mp->media->monologue, NO_RECORDING))
+		return;
 	struct recording *recording = mp->call->recording;
-	mutex_lock(&recording->u.pcap.recording_lock);
+	mutex_lock(&recording->pcap.recording_lock);
 	stream_pcap_dump(mp, s);
-	recording->u.pcap.packet_num++;
-	mutex_unlock(&recording->u.pcap.recording_lock);
+	recording->pcap.packet_num++;
+	mutex_unlock(&recording->pcap.recording_lock);
 }
 
-static void finish_pcap(struct call *call) {
+static void finish_pcap(struct call *call, bool discard) {
 	rec_pcap_recording_finish_file(call->recording);
-	rec_pcap_meta_finish_file(call);
+	if (!discard)
+		rec_pcap_meta_finish_file(call);
+	else
+		rec_pcap_meta_discard_file(call);
 }
 
 static void response_pcap(struct recording *recording, bencode_item_t *output) {
-	if (!recording->u.pcap.recording_path)
+	if (!recording->pcap.recording_path)
 		return;
 
 	bencode_item_t *recordings = bencode_dictionary_add_list(output, "recordings");
-	bencode_list_add_string(recordings, recording->u.pcap.recording_path);
+	bencode_list_add_string(recordings, recording->pcap.recording_path);
 }
 
 
@@ -660,19 +725,18 @@ static void response_pcap(struct recording *recording, bencode_item_t *output) {
 
 
 
-void recording_finish(struct call *call) {
+void recording_finish(struct call *call, bool discard) {
 	if (!call || !call->recording)
 		return;
 
-	__call_unkernelize(call);
+	__call_unkernelize(call, "recording finished");
 
 	struct recording *recording = call->recording;
 
-	_rm(finish, call);
+	_rm(finish, call, discard);
 
-	free(recording->meta_prefix);
-	free(recording->escaped_callid);
-	free(recording->meta_filepath);
+	g_clear_pointer(&recording->meta_prefix, g_free);
+	g_clear_pointer(&recording->escaped_callid, free);
 
 	g_slice_free1(sizeof(*(recording)), recording);
 	call->recording = NULL;
@@ -687,10 +751,10 @@ void recording_finish(struct call *call) {
 
 static int open_proc_meta_file(struct recording *recording) {
 	int fd;
-	fd = open(recording->meta_filepath, O_WRONLY | O_APPEND | O_CREAT, 0666);
+	fd = open(recording->proc.meta_filepath, O_WRONLY | O_APPEND | O_CREAT, 0666);
 	if (fd == -1) {
 		ilog(LOG_ERR, "Failed to open recording metadata file '%s' for writing: %s",
-				recording->meta_filepath, strerror(errno));
+				recording->proc.meta_filepath, strerror(errno));
 		return -1;
 	}
 	return fd;
@@ -744,20 +808,20 @@ static int append_meta_chunk(struct recording *recording, const char *buf, unsig
 static void proc_init(struct call *call) {
 	struct recording *recording = call->recording;
 
-	recording->u.proc.call_idx = UNINIT_IDX;
+	recording->proc.call_idx = UNINIT_IDX;
 	if (!kernel.is_open) {
 		ilog(LOG_WARN, "Call recording through /proc interface requested, but kernel table not open");
 		return;
 	}
-	recording->u.proc.call_idx = kernel_add_call(recording->meta_prefix);
-	if (recording->u.proc.call_idx == UNINIT_IDX) {
+	recording->proc.call_idx = kernel_add_call(recording->meta_prefix);
+	if (recording->proc.call_idx == UNINIT_IDX) {
 		ilog(LOG_ERR, "Failed to add call to kernel recording interface: %s", strerror(errno));
 		return;
 	}
-	ilog(LOG_DEBUG, "kernel call idx is %u", recording->u.proc.call_idx);
+	ilog(LOG_DEBUG, "kernel call idx is %u", recording->proc.call_idx);
 
-	recording->meta_filepath = file_path_str(recording->meta_prefix, "/", ".meta");
-	unlink(recording->meta_filepath); // start fresh XXX good idea?
+	recording->proc.meta_filepath = file_path_str(recording->meta_prefix, "/", ".meta");
+	unlink(recording->proc.meta_filepath); // start fresh XXX good idea?
 
 	append_meta_chunk_str(recording, &call->callid, "CALL-ID");
 	append_meta_chunk_s(recording, recording->meta_prefix, "PARENT");
@@ -772,30 +836,49 @@ static void sdp_before_proc(struct recording *recording, const str *sdp, struct
 			"SDP from %u before %s", ml->unique_id, get_opmode_text(opmode));
 }
 
-static void sdp_after_proc(struct recording *recording, GString *str, struct call_monologue *ml,
+static void sdp_after_proc(struct recording *recording, GString *s, struct call_monologue *ml,
 		enum call_opmode opmode)
 {
-	append_meta_chunk(recording, str->str, str->len,
+	append_meta_chunk(recording, s->str, s->len,
 			"SDP from %u after %s", ml->unique_id, get_opmode_text(opmode));
 }
 
-static void finish_proc(struct call *call) {
+static void finish_proc(struct call *call, bool discard) {
 	struct recording *recording = call->recording;
 	if (!kernel.is_open)
 		return;
-	if (recording->u.proc.call_idx != UNINIT_IDX) {
-		kernel_del_call(recording->u.proc.call_idx);
-		recording->u.proc.call_idx = UNINIT_IDX;
+	if (recording->proc.call_idx != UNINIT_IDX) {
+		kernel_del_call(recording->proc.call_idx);
+		recording->proc.call_idx = UNINIT_IDX;
 	}
 	for (GList *l = call->streams.head; l; l = l->next) {
 		struct packet_stream *ps = l->data;
-		ps->recording.u.proc.stream_idx = UNINIT_IDX;
+		ps->recording.proc.stream_idx = UNINIT_IDX;
 	}
-	unlink(recording->meta_filepath);
+
+	const char *unlink_fn = recording->proc.meta_filepath;
+	AUTO_CLEANUP_GBUF(discard_fn);
+	if (discard) {
+		discard_fn = g_strdup_printf("%s.DISCARD", recording->proc.meta_filepath);
+		int ret = rename(recording->proc.meta_filepath, discard_fn);
+		if (ret)
+			ilog(LOG_ERR, "Failed to rename metadata file \"%s\" to \"%s\": %s",
+					recording->proc.meta_filepath,
+					discard_fn,
+					strerror(errno));
+		unlink_fn = discard_fn;
+	}
+
+	int ret = unlink(unlink_fn);
+	if (ret)
+		ilog(LOG_ERR, "Failed to delete metadata file \"%s\": %s",
+				unlink_fn, strerror(errno));
+
+	g_clear_pointer(&recording->proc.meta_filepath, free);
 }
 
 static void init_stream_proc(struct packet_stream *stream) {
-	stream->recording.u.proc.stream_idx = UNINIT_IDX;
+	stream->recording.proc.stream_idx = UNINIT_IDX;
 }
 
 static void setup_stream_proc(struct packet_stream *stream) {
@@ -810,7 +893,9 @@ static void setup_stream_proc(struct packet_stream *stream) {
 		return;
 	if (!kernel.is_open)
 		return;
-	if (stream->recording.u.proc.stream_idx != UNINIT_IDX)
+	if (stream->recording.proc.stream_idx != UNINIT_IDX)
+		return;
+	if (ML_ISSET(ml, NO_RECORDING))
 		return;
 
 	len = snprintf(buf, sizeof(buf), "TAG %u MEDIA %u TAG-MEDIA %u COMPONENT %u FLAGS %u",
@@ -822,12 +907,12 @@ static void setup_stream_proc(struct packet_stream *stream) {
 			ml->unique_id, media->index, stream->component,
 			(PS_ISSET(stream, RTCP) && !PS_ISSET(stream, RTP)) ? "RTCP" : "RTP",
 			stream->unique_id);
-	stream->recording.u.proc.stream_idx = kernel_add_intercept_stream(recording->u.proc.call_idx, buf);
-	if (stream->recording.u.proc.stream_idx == UNINIT_IDX) {
+	stream->recording.proc.stream_idx = kernel_add_intercept_stream(recording->proc.call_idx, buf);
+	if (stream->recording.proc.stream_idx == UNINIT_IDX) {
 		ilog(LOG_ERR, "Failed to add stream to kernel recording interface: %s", strerror(errno));
 		return;
 	}
-	ilog(LOG_DEBUG, "kernel stream idx is %u", stream->recording.u.proc.stream_idx);
+	ilog(LOG_DEBUG, "kernel stream idx is %u", stream->recording.proc.stream_idx);
 	append_meta_chunk(recording, buf, len, "STREAM %u interface", stream->unique_id);
 }
 
@@ -837,6 +922,8 @@ static void setup_monologue_proc(struct call_monologue *ml) {
 
 	if (!recording)
 		return;
+	if (ML_ISSET(ml, NO_RECORDING))
+		return;
 
 	append_meta_chunk_str(recording, &ml->tag, "TAG %u", ml->unique_id);
 	if (ml->label.len)
@@ -851,6 +938,8 @@ static void setup_media_proc(struct call_media *media) {
 
 	if (!recording)
 		return;
+	if (ML_ISSET(media->monologue, NO_RECORDING))
+		return;
 
 	append_meta_chunk_null(recording, "MEDIA %u PTIME %i", media->unique_id, media->ptime);
 
@@ -871,20 +960,19 @@ static void setup_media_proc(struct call_media *media) {
 
 static void dump_packet_proc(struct media_packet *mp, const str *s) {
 	struct packet_stream *stream = mp->stream;
-	if (stream->recording.u.proc.stream_idx == UNINIT_IDX)
+	if (stream->recording.proc.stream_idx == UNINIT_IDX)
 		return;
 
-	struct rtpengine_message *remsg;
-	unsigned char pkt[sizeof(*remsg) + s->len + MAX_PACKET_HEADER_LEN];
-	remsg = (void *) pkt;
+	struct rtpengine_command_packet *cmd;
+	unsigned char pkt[sizeof(*cmd) + s->len + MAX_PACKET_HEADER_LEN];
+	cmd = (void *) pkt;
 
-	ZERO(*remsg);
-	remsg->cmd = REMG_PACKET;
-	//remsg->u.packet.call_idx = stream->call->recording->u.proc.call_idx; // unused
-	remsg->u.packet.stream_idx = stream->recording.u.proc.stream_idx;
+	cmd->cmd = REMG_PACKET;
+	//remsg->packet.call_idx = stream->call->recording->proc.call_idx; // unused
+	cmd->packet.stream_idx = stream->recording.proc.stream_idx;
 
-	unsigned int pkt_len = fake_ip_header(remsg->data, mp, s);
-	pkt_len += sizeof(*remsg);
+	unsigned int pkt_len = fake_ip_header(cmd->packet.data, mp, s);
+	pkt_len += sizeof(*cmd);
 
 	int ret = write(kernel.fd, pkt, pkt_len);
 	if (ret < 0)
@@ -894,13 +982,48 @@ static void dump_packet_proc(struct media_packet *mp, const str *s) {
 static void kernel_info_proc(struct packet_stream *stream, struct rtpengine_target_info *reti) {
 	if (!stream->call->recording)
 		return;
-	if (stream->recording.u.proc.stream_idx == UNINIT_IDX)
+	if (stream->recording.proc.stream_idx == UNINIT_IDX)
 		return;
-	ilog(LOG_DEBUG, "enabling kernel intercept with stream idx %u", stream->recording.u.proc.stream_idx);
+	ilog(LOG_DEBUG, "enabling kernel intercept with stream idx %u", stream->recording.proc.stream_idx);
 	reti->do_intercept = 1;
-	reti->intercept_stream_idx = stream->recording.u.proc.stream_idx;
+	reti->intercept_stream_idx = stream->recording.proc.stream_idx;
 }
 
 static void meta_chunk_proc(struct recording *recording, const char *label, const str *data) {
 	append_meta_chunk_str(recording, data, "%s", label);
 }
+
+static int create_spool_dir_all(const char *spoolpath) {
+	int ret1, ret2;
+
+	ret1 = rec_pcap_create_spool_dir(spoolpath);
+	ret2 = check_main_spool_dir(spoolpath);
+
+	if (ret1 == FALSE || ret2 == FALSE) {
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static void init_all(struct call *call) {
+	rec_pcap_init(call);
+	proc_init(call);
+}
+
+static void sdp_after_all(struct recording *recording, GString *s, struct call_monologue *ml,
+		enum call_opmode opmode)
+{
+	sdp_after_pcap(recording, s, ml, opmode);
+	sdp_after_proc(recording, s, ml, opmode);
+}
+
+static void dump_packet_all(struct media_packet *mp, const str *s) {
+	dump_packet_pcap(mp, s);
+	dump_packet_proc(mp, s);
+}
+
+static void finish_all(struct call *call, bool discard) {
+	finish_pcap(call, discard);
+	finish_proc(call, discard);
+}
