@@ -1,6 +1,6 @@
 #include "ssrc.h"
 #include <glib.h>
-#include "aux.h"
+#include "helpers.h"
 #include "call.h"
 #include "rtplib.h"
 #include "codeclib.h"
@@ -120,7 +120,7 @@ static int ssrc_time_cmp(const void *aa, const void *bb, void *pp) {
 }
 
 // returns a new reference
-void *get_ssrc(uint32_t ssrc, struct ssrc_hash *ht /* , int *created */) {
+void *get_ssrc_full(uint32_t ssrc, struct ssrc_hash *ht, bool *created) {
 	struct ssrc_entry *ent;
 
 	if (!ht)
@@ -129,8 +129,8 @@ void *get_ssrc(uint32_t ssrc, struct ssrc_hash *ht /* , int *created */) {
 restart:
 	ent = find_ssrc(ssrc, ht);
 	if (G_LIKELY(ent)) {
-//		if (created)
-//			*created = 0;
+		if (created)
+			*created = false;
 		return ent;
 	}
 
@@ -172,8 +172,8 @@ restart:
 	add_ssrc_entry(ssrc, ent, ht);
 	g_atomic_pointer_set(&ht->cache, ent);
 	rwlock_unlock_w(&ht->lock);
-//	if (created)
-//		*created = 1;
+	if (created)
+		*created = true;
 
 	return ent;
 }
@@ -202,13 +202,17 @@ void ssrc_hash_foreach(struct ssrc_hash *sh, void (*f)(void *, void *), void *pt
 }
 
 
-struct ssrc_hash *create_ssrc_hash_full(ssrc_create_func_t cfunc, void *uptr) {
+struct ssrc_hash *create_ssrc_hash_full_fast(ssrc_create_func_t cfunc, void *uptr) {
 	struct ssrc_hash *ret;
 	ret = g_slice_alloc0(sizeof(*ret));
 	ret->ht = g_hash_table_new_full(uint32_hash, uint32_eq, NULL, ssrc_entry_put);
 	rwlock_init(&ret->lock);
 	ret->create_func = cfunc;
 	ret->uptr = uptr;
+	return ret;
+}
+struct ssrc_hash *create_ssrc_hash_full(ssrc_create_func_t cfunc, void *uptr) {
+	struct ssrc_hash *ret = create_ssrc_hash_full_fast(cfunc, uptr);
 	ret->precreat = cfunc(uptr); // because object creation might be slow
 	return ret;
 }
@@ -351,7 +355,7 @@ void ssrc_sender_report(struct call_media *m, const struct ssrc_sender_report *s
 	mutex_unlock(&e->lock);
 	obj_put(e);
 }
-void ssrc_receiver_report(struct call_media *m, const struct ssrc_receiver_report *rr,
+void ssrc_receiver_report(struct call_media *m, struct stream_fd *sfd, const struct ssrc_receiver_report *rr,
 		const struct timeval *tv)
 {
 	ilog(LOG_DEBUG, "RR from %s%x%s about %s%x%s: FL %u TL %u HSR %u J %u LSR %u DLSR %u",
@@ -395,14 +399,18 @@ void ssrc_receiver_report(struct call_media *m, const struct ssrc_receiver_repor
 		.reported = *tv,
 		.packetloss = (unsigned int) rr->fraction_lost * 100 / 256,
 	};
+
+	RTPE_SAMPLE_SFD(jitter, jitter, sfd);
+	RTPE_SAMPLE_SFD(rtt_e2e, rtt_end2end, sfd);
+	RTPE_SAMPLE_SFD(rtt_dsct, rtt, sfd);
+	RTPE_SAMPLE_SFD(packetloss, ssb->packetloss, sfd);
+
 	other_e->packets_lost = rr->packets_lost;
 	mos_calc(ssb);
 	if (ssb->mos) {
 		ilog(LOG_DEBUG, "Calculated MOS from RR for %s%x%s is %.1f", FMT_M(rr->from),
 				(double) ssb->mos / 10.0);
-		RTPE_STATS_ADD(mos, ssb->mos);
-		RTPE_STATS_ADD(mos2, ssb->mos * ssb->mos);
-		RTPE_STATS_INC(mos_num);
+		RTPE_SAMPLE_SFD(mos, ssb->mos, sfd);
 	}
 
 	// got a new stats block, add it to reporting ssrc
@@ -570,9 +578,7 @@ void payload_tracker_add(struct payload_tracker *t, int pt) {
 	}
 
 	// fill in new entry
-	t->last[t->last_idx++] = pt;
-	if (t->last_idx >= G_N_ELEMENTS(t->last))
-		t->last_idx = 0;
+	t->last[t->last_idx] = pt;
 
 	// increase new counter
 	PT_DBG("increasing new pt count from %u", t->count[pt]);
@@ -593,5 +599,34 @@ void payload_tracker_add(struct payload_tracker *t, int pt) {
 		__pt_sort(t, old_pt);
 
 out:
+	if (++t->last_idx >= G_N_ELEMENTS(t->last))
+		t->last_idx = 0;
 	mutex_unlock(&t->lock);
 }
+
+
+// call master lock held in R
+void ssrc_collect_metrics(struct call_media *media) {
+	if (!media->streams.head)
+		return;
+	struct packet_stream *ps = media->streams.head->data;
+	for (int i = 0; i < RTPE_NUM_SSRC_TRACKING; i++) {
+		struct ssrc_ctx *s = ps->ssrc_in[i];
+		if (!s)
+			break; // end of list
+		struct ssrc_entry_call *e = s->parent;
+
+		// exclude zero values - technically possible but unlikely and probably just unset
+		if (!e->jitter)
+			continue;
+
+		if (e->input_ctx.tracker.most_len > 0 && e->input_ctx.tracker.most[0] != 255) {
+			const struct rtp_payload_type *rpt = rtp_payload_type(e->input_ctx.tracker.most[0],
+					&ps->media->codecs);
+			if (rpt && rpt->clock_rate)
+				e->jitter = e->jitter * 1000 / rpt->clock_rate;
+		}
+
+		RTPE_SAMPLE_SFD(jitter_measured, e->jitter, ps->selected_sfd);
+	}
+}
