@@ -265,6 +265,24 @@ static const char *__websocket_write_http_response(struct websocket_conn *wc, in
 	if (content_length >= 0)
 		if (lws_add_http_header_content_length(wc->wsi, content_length, &p, end))
 			return "Failed to add HTTP headers to response";
+
+	if (lws_add_http_header_by_name(wc->wsi,
+				(unsigned char *) "Access-Control-Allow-Origin:",
+				(unsigned char *) "*", 1, &p, end))
+		return "Failed to add CORS header";
+	if (lws_add_http_header_by_name(wc->wsi,
+				(unsigned char *) "Access-Control-Max-Age:",
+				(unsigned char *) "86400", 5, &p, end))
+		return "Failed to add CORS header";
+	if (lws_add_http_header_by_name(wc->wsi,
+				(unsigned char *) "Access-Control-Allow-Methods:",
+				(unsigned char *) "GET, POST", 9, &p, end))
+		return "Failed to add CORS header";
+	if (lws_add_http_header_by_name(wc->wsi,
+				(unsigned char *) "Access-Control-Allow-Headers:",
+				(unsigned char *) "Content-Type", 12, &p, end))
+		return "Failed to add CORS header";
+
 	if (lws_finalize_http_header(wc->wsi, &p, end))
 		return "Failed to write HTTP headers";
 
@@ -301,7 +319,7 @@ static int websocket_dequeue(struct websocket_conn *wc) {
 		g_string_set_size(wo->str, wo->str->len + LWS_SEND_BUFFER_POST_PADDING);
 		size_t to_send = wo->str->len - wo->str_done - LWS_SEND_BUFFER_POST_PADDING;
 		if (to_send) {
-			if (to_send > 2000)
+			if (to_send > 10000)
 				ilogs(http, LOG_DEBUG, "Writing %lu bytes to LWS", (unsigned long) to_send);
 			else
 				ilogs(http, LOG_DEBUG, "Writing back to LWS: '%.*s'",
@@ -362,7 +380,7 @@ static const char *websocket_http_ping(struct websocket_message *wm) {
 static const char *websocket_http_metrics(struct websocket_message *wm) {
 	ilogs(http, LOG_DEBUG, "Respoding to GET /metrics");
 
-	AUTO_CLEANUP_INIT(GQueue *metrics, statistics_free_metrics, statistics_gather_metrics());
+	AUTO_CLEANUP_INIT(GQueue *metrics, statistics_free_metrics, statistics_gather_metrics(NULL));
 	AUTO_CLEANUP_INIT(GString *outp, __g_string_free, g_string_new(""));
 	AUTO_CLEANUP_INIT(GHashTable *metric_types, __g_hash_table_destroy,
 			g_hash_table_new(g_str_hash, g_str_equal));
@@ -414,8 +432,7 @@ static const char *websocket_http_cli(struct websocket_message *wm) {
 
 	ilogs(http, LOG_DEBUG, "Respoding to GET /cli/%s", uri);
 
-	str uri_cmd;
-	str_init(&uri_cmd, uri);
+	str uri_cmd = STR_INIT(uri);
 
 	struct cli_writer cw = {
 		.cw_printf = websocket_queue_printf,
@@ -433,8 +450,7 @@ static const char *websocket_http_cli(struct websocket_message *wm) {
 static const char *websocket_cli_process(struct websocket_message *wm) {
 	ilogs(http, LOG_DEBUG, "Processing websocket CLI req '%s'", wm->body->str);
 
-	str uri_cmd;
-	str_init_len(&uri_cmd, wm->body->str, wm->body->len);
+	str uri_cmd = STR_INIT_LEN(wm->body->str, wm->body->len);
 
 	struct cli_writer cw = {
 		.cw_printf = websocket_queue_printf,
@@ -447,14 +463,18 @@ static const char *websocket_cli_process(struct websocket_message *wm) {
 }
 
 
-static void websocket_ng_send_ws(str *cookie, str *body, const endpoint_t *sin, void *p1) {
+static void websocket_ng_send_ws(str *cookie, str *body, const endpoint_t *sin, const sockaddr_t *from,
+		void *p1)
+{
 	struct websocket_conn *wc = p1;
 	websocket_queue_raw(wc, cookie->s, cookie->len);
 	websocket_queue_raw(wc, " ", 1);
 	websocket_queue_raw(wc, body->s, body->len);
 	websocket_write_binary(wc, NULL, 0, true);
 }
-static void websocket_ng_send_http(str *cookie, str *body, const endpoint_t *sin, void *p1) {
+static void websocket_ng_send_http(str *cookie, str *body, const endpoint_t *sin, const sockaddr_t *from,
+		void *p1)
+{
 	struct websocket_conn *wc = p1;
 	websocket_http_response(wc, 200, "application/x-rtpengine-ng", cookie->len + 1 + body->len);
 	websocket_queue_raw(wc, cookie->s, cookie->len);
@@ -481,7 +501,7 @@ static const char *websocket_ng_process(struct websocket_message *wm) {
 	str_init_len(&buf->cmd, buf->body->str, buf->body->len);
 	buf->endpoint = wm->wc->endpoint;
 
-	control_ng_process(&buf->cmd, &buf->endpoint, buf->addr, websocket_ng_send_ws, wm->wc, &buf->obj);
+	control_ng_process(&buf->cmd, &buf->endpoint, buf->addr, NULL, websocket_ng_send_ws, wm->wc, &buf->obj);
 
 	obj_put(buf);
 
@@ -500,7 +520,8 @@ static const char *websocket_http_ng(struct websocket_message *wm) {
 	str_init_len(&buf->cmd, buf->body->str, buf->body->len);
 	buf->endpoint = wm->wc->endpoint;
 
-	if (control_ng_process(&buf->cmd, &buf->endpoint, buf->addr, websocket_ng_send_http, wm->wc, &buf->obj))
+	if (control_ng_process(&buf->cmd, &buf->endpoint, buf->addr, NULL, websocket_ng_send_http, wm->wc,
+				&buf->obj))
 		websocket_http_complete(wm->wc, 600, "text/plain", 6, "error\n");
 
 	obj_put(buf);
@@ -524,7 +545,7 @@ static int websocket_http_get(struct websocket_conn *wc) {
 	const char *uri = wm->uri;
 	websocket_message_func_t handler = NULL;
 
-	ilogs(http, LOG_INFO, "HTTP GET from %s: '%s'", endpoint_print_buf(&wc->endpoint), wm->uri);
+	ilogs(http, LOG_INFO, "HTTP GET from %s: '%s'", endpoint_print_buf(&wc->endpoint), uri);
 	wm->method = M_GET;
 
 	if (!strcmp(uri, "/ping"))
@@ -533,6 +554,8 @@ static int websocket_http_get(struct websocket_conn *wc) {
 		handler = websocket_http_cli;
 	else if (!strcmp(uri, "/metrics"))
 		handler = websocket_http_metrics;
+	else if (!strncmp(uri, "/admin/", 7))
+		handler = websocket_janus_get;
 	else
 		handler = websocket_http_404;
 
@@ -576,6 +599,25 @@ static int websocket_http_post(struct websocket_conn *wc) {
 }
 
 
+static const char *websocket_http_options_generic(struct websocket_message *wm) {
+	ilogs(http, LOG_DEBUG, "Respoding to OPTIONS");
+	websocket_http_complete(wm->wc, 200, NULL, 0, NULL);
+	return NULL;
+}
+
+
+static int websocket_http_options(struct websocket_conn *wc) {
+	struct websocket_message *wm = wc->wm;
+
+	ilogs(http, LOG_INFO, "HTTP OPTIONS from %s: '%s'", endpoint_print_buf(&wc->endpoint), wm->uri);
+	wm->method = M_OPTIONS;
+
+	websocket_message_push(wc, websocket_http_options_generic);
+
+	return 0;
+}
+
+
 static int websocket_http_body(struct websocket_conn *wc, const char *body, size_t len) {
 	struct websocket_message *wm = wc->wm;
 	const char *uri = wm->uri;
@@ -598,6 +640,10 @@ static int websocket_http_body(struct websocket_conn *wc, const char *body, size
 		handler = websocket_http_ng;
 	else if (!strcmp(uri, "/admin") && wm->method == M_POST && wm->content_type == CT_JSON)
 		handler = websocket_janus_process;
+	else if (!strcmp(uri, "/janus") && wm->method == M_POST && wm->content_type == CT_JSON)
+		handler = websocket_janus_process;
+	else if (!strncmp(uri, "/janus/", 7) && wm->method == M_POST && wm->content_type == CT_JSON)
+		handler = websocket_janus_post;
 	else
 		handler = websocket_http_404;
 
@@ -723,6 +769,8 @@ static int websocket_do_http(struct lws *wsi, struct websocket_conn *wc, const c
 		return websocket_http_get(wc);
 	if (lws_hdr_total_length(wsi, WSI_TOKEN_POST_URI))
 		return websocket_http_post(wc);
+	if (lws_hdr_total_length(wsi, WSI_TOKEN_OPTIONS_URI))
+		return websocket_http_options(wc);
 
 	ilogs(http, LOG_INFO, "Ignoring HTTP request to %s with unsupported method", uri);
 	return 0;
