@@ -11,7 +11,6 @@
 #include <glib.h>
 #include <time.h>
 #include <sys/time.h>
-#include <pcre.h>
 #include <openssl/x509.h>
 #include <limits.h>
 #include <stdbool.h>
@@ -60,6 +59,7 @@ enum call_opmode {
 	OP_REQUEST,
 	OP_REQ_ANSWER,
 	OP_PUBLISH,
+	OP_PLAY_MEDIA,
 	OP_OTHER,
 };
 
@@ -87,7 +87,6 @@ enum {
 
 #define ERROR_NO_FREE_PORTS	-100
 #define ERROR_NO_FREE_LOGS	-101
-#define ERROR_NO_ICE_AGENT	-102
 
 #ifndef RTP_LOOP_PROTECT
 #define RTP_LOOP_PROTECT	28 /* number of bytes */
@@ -95,7 +94,7 @@ enum {
 #define RTP_LOOP_MAX_COUNT	30 /* number of consecutively detected dupes to trigger protection */
 #endif
 
-#define IS_FOREIGN_CALL(c) (c->foreign_call)
+#define IS_FOREIGN_CALL(c) CALL_ISSET(c, FOREIGN)
 #define IS_OWN_CALL(c) !IS_FOREIGN_CALL(c)
 
 /* flags shared by several of the structs below */
@@ -113,6 +112,8 @@ enum {
 #define SHARED_FLAG_ICE_LITE_PEER		0x00000800
 #define SHARED_FLAG_UNIDIRECTIONAL		0x00001000
 #define SHARED_FLAG_RTCP_FB			0x00002000
+#define SHARED_FLAG_LEGACY_OSRTP		0x00004000
+#define SHARED_FLAG_LEGACY_OSRTP_REV		0x00008000
 
 /* struct stream_params */
 #define SP_FLAG_NO_RTCP				0x00010000
@@ -130,6 +131,8 @@ enum {
 #define SP_FLAG_TRICKLE_ICE			SHARED_FLAG_TRICKLE_ICE
 #define SP_FLAG_ICE_LITE_PEER			SHARED_FLAG_ICE_LITE_PEER
 #define SP_FLAG_RTCP_FB				SHARED_FLAG_RTCP_FB
+#define SP_FLAG_LEGACY_OSRTP			SHARED_FLAG_LEGACY_OSRTP
+#define SP_FLAG_LEGACY_OSRTP_REV		SHARED_FLAG_LEGACY_OSRTP_REV
 
 /* struct packet_stream */
 #define PS_FLAG_RTP				0x00010000
@@ -175,7 +178,6 @@ enum {
 #define MEDIA_FLAG_ICE_LITE_PEER		SHARED_FLAG_ICE_LITE_PEER
 #define MEDIA_FLAG_ICE_CONTROLLING		0x00200000
 #define MEDIA_FLAG_LOOP_CHECK			0x00400000
-#define MEDIA_FLAG_TRANSCODE			0x00800000
 #define MEDIA_FLAG_PTIME_OVERRIDE		0x01000000
 #define MEDIA_FLAG_RTCP_FB			SHARED_FLAG_RTCP_FB
 #define MEDIA_FLAG_GENERATOR			0x02000000
@@ -183,6 +185,37 @@ enum {
 #define MEDIA_FLAG_RTCP_GEN			0x08000000
 #define MEDIA_FLAG_ECHO				0x10000000
 #define MEDIA_FLAG_BLACKHOLE			0x20000000
+#define MEDIA_FLAG_REORDER_FORCED		0x40000000
+#define MEDIA_FLAG_AUDIO_PLAYER			0x80000000
+#define MEDIA_FLAG_LEGACY_OSRTP			SHARED_FLAG_LEGACY_OSRTP
+#define MEDIA_FLAG_LEGACY_OSRTP_REV		SHARED_FLAG_LEGACY_OSRTP_REV
+
+/* struct call_monologue */
+#define ML_FLAG_REC_FORWARDING			0x00010000
+#define ML_FLAG_INJECT_DTMF			0x00020000
+#define ML_FLAG_DTMF_INJECTION_ACTIVE		0x00040000
+#define ML_FLAG_DETECT_DTMF			0x00080000
+#define ML_FLAG_NO_RECORDING			0x00100000
+#define ML_FLAG_TRANSCODING			0x00200000
+#define ML_FLAG_BLOCK_SHORT			0x00400000
+#define ML_FLAG_BLOCK_MEDIA			0x00800000
+#define ML_FLAG_SILENCE_MEDIA			0x01000000
+
+/* struct call */
+#define CALL_FLAG_IPV4_OFFER			0x00010000
+#define CALL_FLAG_IPV6_OFFER			0x00020000
+#define CALL_FLAG_IPV4_ANSWER			0x00040000
+#define CALL_FLAG_IPV6_ANSWER			0x00080000
+#define CALL_FLAG_MEDIA_COUNTED			0x00100000
+#define CALL_FLAG_RECORDING_ON			0x00200000
+#define CALL_FLAG_REC_FORWARDING		0x00400000
+#define CALL_FLAG_DROP_TRAFFIC			0x00800000
+#define CALL_FLAG_FOREIGN			0x01000000 // created_via_redis_notify call
+#define CALL_FLAG_FOREIGN_MEDIA			0x02000000 // for calls taken over, tracks whether we have media
+#define CALL_FLAG_DISABLE_JB			0x04000000
+#define CALL_FLAG_DEBUG				0x08000000
+#define CALL_FLAG_BLOCK_MEDIA			0x10000000
+#define CALL_FLAG_SILENCE_MEDIA			0x20000000
 
 /* access macros */
 #define SP_ISSET(p, f)		bf_isset(&(p)->sp_flags, SP_FLAG_ ## f)
@@ -198,6 +231,16 @@ enum {
 #define MEDIA_ARESET2(p, f, g)	bf_areset(&(p)->media_flags, MEDIA_FLAG_ ## f | MEDIA_FLAG_ ## g)
 #define MEDIA_SET(p, f)		bf_set(&(p)->media_flags, MEDIA_FLAG_ ## f)
 #define MEDIA_CLEAR(p, f)	bf_clear(&(p)->media_flags, MEDIA_FLAG_ ## f)
+#define ML_ISSET(p, f)		bf_isset(&(p)->ml_flags, ML_FLAG_ ## f)
+#define ML_ISSET2(p, f, g)	bf_isset(&(p)->ml_flags, ML_FLAG_ ## f | ML_FLAG_ ## g)
+#define ML_ARESET2(p, f, g)	bf_areset(&(p)->ml_flags, ML_FLAG_ ## f | ML_FLAG_ ## g)
+#define ML_SET(p, f)		bf_set(&(p)->ml_flags, ML_FLAG_ ## f)
+#define ML_CLEAR(p, f)		bf_clear(&(p)->ml_flags, ML_FLAG_ ## f)
+#define CALL_ISSET(p, f)		bf_isset(&(p)->call_flags, CALL_FLAG_ ## f)
+#define CALL_ISSET2(p, f, g)	bf_isset(&(p)->call_flags, CALL_FLAG_ ## f | CALL_FLAG_ ## g)
+#define CALL_ARESET2(p, f, g)	bf_areset(&(p)->call_flags, CALL_FLAG_ ## f | CALL_FLAG_ ## g)
+#define CALL_SET(p, f)		bf_set(&(p)->call_flags, CALL_FLAG_ ## f)
+#define CALL_CLEAR(p, f)		bf_clear(&(p)->call_flags, CALL_FLAG_ ## f)
 
 enum block_dtmf_mode {
 	BLOCK_DTMF_OFF = 0,
@@ -225,7 +268,6 @@ enum block_dtmf_mode {
 #include "dtls.h"
 
 
-struct poller;
 struct control_stream;
 struct call;
 struct redis;
@@ -246,6 +288,7 @@ struct codec_tracker;
 struct rtcp_timer;
 struct mqtt_timer;
 struct janus_session;
+struct audio_player;
 
 
 typedef bencode_buffer_t call_buffer_t;
@@ -290,6 +333,7 @@ struct stream_params {
 	int			ptime;
 	str			media_id;
 	struct t38_options	t38_options;
+	str			tls_id;
 };
 
 struct endpoint_map {
@@ -308,12 +352,27 @@ struct loop_protector {
 
 
 
+/**
+ * The packet_stream itself can be marked as:
+ * - SRTP endpoint
+ * - ICE endpoint
+ * - send/receive-only
+ * 
+ * This is done through the various bit flags.
+ */
 struct packet_stream {
-	mutex_t			in_lock,
-				out_lock;
 	/* Both locks valid only with call->master_lock held in R.
 	 * Preempted by call->master_lock held in W.
-	 * If both in/out are to be locked, in_lock must be locked first. */
+	 * If both in/out are to be locked, in_lock must be locked first.
+	 * 
+	 * The in_lock protects fields relevant to packet reception on that stream,
+	 * meanwhile the out_lock protects fields relevant to packet egress.
+	 * 
+	 * This allows packet handling on multiple ports and streams belonging
+	 * to the same call to happen at the same time.
+	 */
+	mutex_t			in_lock,
+				out_lock;
 
 	struct call_media	*media;		/* RO */
 	struct call		*call;		/* RO */
@@ -325,29 +384,31 @@ struct packet_stream {
 	struct stream_fd *	selected_sfd;
 	endpoint_t		last_local_endpoint;
 	struct dtls_connection	ice_dtls;	/* LOCK: in_lock */
-	GQueue			rtp_sinks;	// LOCK: call->master_lock, in_lock for streamhandler
-	GQueue			rtcp_sinks;	// LOCK: call->master_lock, in_lock for streamhandler
+	GQueue			rtp_sinks;	/* LOCK: call->master_lock, in_lock for streamhandler */
+	GQueue			rtcp_sinks;	/* LOCK: call->master_lock, in_lock for streamhandler */
 	struct packet_stream	*rtcp_sibling;	/* LOCK: call->master_lock */
-	GQueue			rtp_mirrors;	// LOCK: call->master_lock, in_lock for streamhandler
+	GQueue			rtp_mirrors;	/* LOCK: call->master_lock, in_lock for streamhandler */
 	struct endpoint		endpoint;	/* LOCK: out_lock */
-	struct endpoint		detected_endpoints[4];	/* LOCK: out_lock */
-	struct timeval		ep_detect_signal; /* LOCK: out_lock */
-	struct endpoint		advertised_endpoint; /* RO */
-	struct endpoint		learned_endpoint; /* LOCK: out_lock */
-	struct crypto_context	crypto;		/* OUT direction, LOCK: out_lock */
+	struct endpoint		detected_endpoints[4];		/* LOCK: out_lock */
+	struct timeval		ep_detect_signal;		/* LOCK: out_lock */
+	struct endpoint		advertised_endpoint;		/* RO */
+	struct endpoint		learned_endpoint;		/* LOCK: out_lock */
+	struct crypto_context	crypto;				/* OUT direction, LOCK: out_lock */
 	struct ssrc_ctx		*ssrc_in[RTPE_NUM_SSRC_TRACKING],	/* LOCK: in_lock */
 				*ssrc_out[RTPE_NUM_SSRC_TRACKING];	/* LOCK: out_lock */
-	unsigned int		ssrc_in_idx,	// LOCK: in_lock
-				ssrc_out_idx;	// LOCK: out_lock
-	struct send_timer	*send_timer;	/* RO */
-	struct jitter_buffer	*jb;		/* RO */
+	unsigned int		ssrc_in_idx,				/* LOCK: in_lock */
+				ssrc_out_idx;				/* LOCK: out_lock */
+	struct send_timer	*send_timer;				/* RO */
+	struct jitter_buffer	*jb;					/* RO */
 	time_t kernel_time;
 
-	struct stream_stats	stats;
-	struct stream_stats	kernel_stats;
+	struct stream_stats	stats_in;
+	struct stream_stats	stats_out;
+	struct stream_stats	kernel_stats_in;
+	struct stream_stats	kernel_stats_out;
 	unsigned char		in_tos_tclass;
 	atomic64		last_packet;
-	GHashTable		*rtp_stats;	/* LOCK: call->master_lock */
+	GHashTable		*rtp_stats;				/* LOCK: call->master_lock */
 	struct rtp_stats	*rtp_stats_cache;
 	unsigned int		stats_flags;
 	enum endpoint_learning		el_flags;
@@ -359,19 +420,24 @@ struct packet_stream {
 	unsigned int		lp_count;
 #endif
 
-	X509			*dtls_cert;	/* LOCK: in_lock */
+	X509			*dtls_cert;				/* LOCK: in_lock */
 
 	/* in_lock must be held for SETTING these: */
 	volatile unsigned int	ps_flags;
 };
 
-/* protected by call->master_lock, except the RO elements */
+/**
+ * Protected by call->master_lock, except the RO elements.
+ * 
+ * call_media is not reference-counted and is completely owned by the call object.
+ * Therefore call_media is released when the call is destroyed.
+ */
 struct call_media {
-	struct call_monologue	*monologue;	/* RO */
-	struct call		*call;		/* RO */
+	struct call_monologue	*monologue;			/* RO */
+	struct call		*call;				/* RO */
 
-	unsigned int		index;		/* RO */
-	unsigned int		unique_id;	/* RO */
+	unsigned int		index;				/* RO */
+	unsigned int		unique_id;			/* RO */
 	str			type;
 	enum media_type		type_id;
 	str			protocol_str;
@@ -385,80 +451,95 @@ struct call_media {
 	str			media_id;
 	str			label;
 	GQueue			sdes_in, sdes_out;
-	struct dtls_fingerprint fingerprint; /* as received */
-	const struct dtls_hash_func *fp_hash_func; // outgoing
+	struct dtls_fingerprint fingerprint;			/* as received */
+	const struct dtls_hash_func *fp_hash_func;		/* outgoing */
+	str			tls_id;
 
-	GQueue			streams; /* normally RTP + RTCP */
+	GQueue			streams;			/* normally RTP + RTCP */
 	GQueue			endpoint_maps;
 
 	struct codec_store	codecs;
-	GQueue			sdp_attributes; // str_sprintf()
-	GHashTable		*codec_handlers; // int payload type -> struct codec_handler
-						// XXX combine this with 'codecs' hash table?
-	GQueue			codec_handlers_store; // storage for struct codec_handler
+	GQueue			sdp_attributes;			/* str_sprintf() */
+	GHashTable		*codec_handlers;		/* int payload type -> struct codec_handler
+														XXX combine this with 'codecs' hash table? */
+	GQueue			codec_handlers_store;		/* storage for struct codec_handler */
 	struct codec_handler	*codec_handler_cache;
 	struct rtcp_handler	*rtcp_handler;
-	struct rtcp_timer	*rtcp_timer;	// master lock for scheduling purposes
-	struct mqtt_timer	*mqtt_timer;	// master lock for scheduling purposes
+	struct rtcp_timer	*rtcp_timer;			/* master lock for scheduling purposes */
+	struct mqtt_timer	*mqtt_timer;			/* master lock for scheduling purposes */
 	//struct codec_handler	*dtmf_injector;
 	struct t38_gateway	*t38_gateway;
+	struct audio_player	*audio_player;
 	struct codec_handler	*t38_handler;
 
 	unsigned int		buffer_delay;
 
 	mutex_t			dtmf_lock;
-	unsigned long		dtmf_ts; // TS of last processed end event
+	unsigned long		dtmf_ts;			/* TS of last processed end event */
+	unsigned int		dtmf_count;
 	// lists are append-only
 	GQueue			dtmf_recv;
 	GQueue			dtmf_send;
 
 #ifdef WITH_TRANSCODING
-	union {
-		struct {
-			struct amr_cmr cmr;
-		} amr;
-	} u;
+	encoder_callback_t	encoder_callback;
 #endif
 
-	int			ptime; // either from SDP or overridden
+	int			ptime;				/* either from SDP or overridden */
 
 	volatile unsigned int	media_flags;
 };
 
-// link between subscribers and subscriptions
+/** 
+ * Link between subscribers and subscriptions.
+ * 
+ * Contain flags and attributes, which can be used
+ * to mark a subscription (for example, as an egress subscription).
+ * 
+ * During signalling events, the list of subscriptions for each call_monologue
+ * is used to create the list of rtp_sink and rtcp_sink given in each packet_stream.
+ * 
+ * Each entry in these lists is a sink_handler object, which again contains flags and attributes.
+ * Flags from a call_subscription are copied into the sink_handler.
+ */
 struct call_subscription {
 	struct call_monologue	*monologue;
 	GList			*link; // link into the corresponding opposite list
 	unsigned int		media_offset; // 0 if media indexes match up
-	unsigned int		offer_answer:1; // bidirectional, exclusive
-	unsigned int		rtcp_only:1;
-	unsigned int		transcoding:1;
-	unsigned int		egress:1;
+	struct sink_attrs	attrs;
 };
 
-/* half a dialogue */
-/* protected by call->master_lock, except the RO elements */
+/**
+ * Half a dialogue.
+ * Protected by call->master_lock, except the RO elements.
+ * 
+ * call_monologue (call participant) contains a list of subscribers
+ * and subscriptions, which are other call_monologue's.
+ * 
+ * These lists are mutual.
+ * A regular A/B call has two call_monologue objects with each subscribed to the other.
+ */
 struct call_monologue {
-	struct call		*call;		/* RO */
-	unsigned int		unique_id;	/* RO */
+	struct call		*call;			/* RO */
+	unsigned int		unique_id;		/* RO */
 
 	str			tag;
 	str			viabranch;
 	enum tag_type		tagtype;
 	str			label;
-	time_t			created;	/* RO */
+	time_t			created;		/* RO */
 	time_t			deleted;
-	struct timeval		started; /* for CDR */
-	struct timeval		terminated; /* for CDR */
+	struct timeval		started;		/* for CDR */
+	struct timeval		terminated;		/* for CDR */
 	enum termination_reason	term_reason;
+	sockfamily_t		*desired_family;
 	const struct logical_intf *logical_intf;
-	GHashTable		*other_tags;
-	GHashTable		*branches;
-	GQueue			subscriptions; // who am I subscribed to (sources)
-	GHashTable		*subscriptions_ht; // for quick lookup
-	GQueue			subscribers; // who is subscribed to me (sinks)
-	GHashTable		*subscribers_ht; // for quick lookup
-	GQueue			medias;
+	GHashTable 		*associated_tags;
+	GQueue			subscriptions;		/* who am I subscribed to (sources) */
+	GHashTable		*subscriptions_ht;	/* for quick lookup */
+	GQueue			subscribers;		/* who is subscribed to me (sinks) */
+	GHashTable		*subscribers_ht;	/* for quick lookup */
+	GPtrArray		*medias;
 	GHashTable		*media_ids;
 	struct media_player	*player;
 	unsigned long long	sdp_session_id;
@@ -471,10 +552,11 @@ struct call_monologue {
 	char			*sdp_session_name;
 	struct ssrc_hash	*ssrc_hash;
 	str			metadata;
+	struct janus_session	*janus_session;
 
 	// DTMF blocking/replacement stuff:
 	enum block_dtmf_mode	block_dtmf;
-	unsigned int		tone_freq;
+	GArray			*tone_freqs;
 	unsigned int		tone_vol;
 	char			dtmf_digit;
 	str			dtmf_trigger;
@@ -486,11 +568,7 @@ struct call_monologue {
 	unsigned int		block_dtmf_trigger_end_ms;
 	unsigned int		dtmf_delay;
 
-	unsigned int		block_media:1;
-	unsigned int		silence_media:1;
-	unsigned int		rec_forwarding:1;
-	unsigned int		inject_dtmf:1;
-	unsigned int		detect_dtmf:1;
+	volatile unsigned int	ml_flags;
 };
 
 struct call_iterator_list {
@@ -541,25 +619,62 @@ struct call_iterator_entry {
 			mutex_unlock(&rtpe_call_iterators[__which].lock); \
 	} while (0)
 
+/**
+ * struct call is the main parent structure of all call-related objects.
+ * 
+ * The logical object hierarchy under the 'struct call':
+ * call > call_monologue > call_media > packet_stream > stream_fd
+ * 
+ * struct call usually has multiple call_monologue objects.
+ * Meanwhile each sub-object of call, as a parent of own sub-objects,
+ * can also contain multiple child objects.
+ * 
+ * Furthermore, each child object contains a back ptr to its parent object.
+ * 
+ * The parent call object contains one list (as GQueue) for each kind of child object.
+ * These lists are what is used to free and release the child objects
+ * during a call teardown.
+ * Every child object owned by the call is added to its respective list exactly once.
+ * 
+ * Call object is reference-counted through the struct obj.
+ */
 struct call {
+
+	/* struct obj member must always be the first member in a struct.
+	 *
+	 * obj is created with a cleanup handler, see obj_alloc(),
+	 * and this handler is executed whenever the reference count drops to zero.
+	 * 
+	 * References are acquired and released through obj_get() and obj_put()
+	 * (plus some other wrapper functions).
+	 */
 	struct obj		obj;
 
 	mutex_t			buffer_lock;
 	call_buffer_t		buffer;
 
-	/* everything below protected by master_lock */
+	/* master_lock protects the entire call and all the contained objects.
+	 * 
+	 * All the fields and any nested sub-object must:
+	 * - only be accessed with the master_lock held as a read lock
+	 * - only be modified with the master_lock held as a write lock
+	 * 
+	 * Therefore, during signalling events acquire a write-lock,
+	 * and during RTP packets handling acquire a read-lock.
+	 */
 	rwlock_t		master_lock;
-	GQueue			monologues;
-	GQueue			medias;
+
+	/* everything below is protected by the master_lock */
+	GQueue			monologues;	/* call_monologue */
+	GQueue			medias;		/* call_media */
 	GHashTable		*tags;
 	GHashTable		*viabranches;
 	GHashTable		*labels;
 	GQueue			streams;
-	GQueue			stream_fds;
+	GQueue			stream_fds;	/* stream_fd */
 	GQueue			endpoint_maps;
-	struct dtls_cert	*dtls_cert; /* for outgoing */
+	struct dtls_cert	*dtls_cert;	/* for outgoing */
 	struct mqtt_timer	*mqtt_timer;
-	struct janus_session	*janus_session;
 
 	str			callid;
 	struct timeval		created;
@@ -571,6 +686,7 @@ struct call {
 	char			*created_from;
 	sockaddr_t		created_from_addr;
 	sockaddr_t		xmlrpc_callback;
+	endpoint_t		dtmf_log_dest;
 
 	unsigned int		redis_hosted_db;
 
@@ -581,114 +697,78 @@ struct call {
 	int			cpu_affinity;
 	enum block_dtmf_mode	block_dtmf;
 
-	// ipv4/ipv6 media flags
-	unsigned int		is_ipv4_media_offer:1;
-	unsigned int		is_ipv6_media_offer:1;
-	unsigned int		is_ipv4_media_answer:1;
-	unsigned int		is_ipv6_media_answer:1;
-	unsigned int		is_call_media_counted:1;
-
-	unsigned int		block_media:1;
-	unsigned int		silence_media:1;
-	unsigned int		recording_on:1;
-	unsigned int		rec_forwarding:1;
-	unsigned int		drop_traffic:1;
-	unsigned int		foreign_call:1; // created_via_redis_notify call
-	unsigned int		foreign_media:1; // for calls taken over, tracks whether we have media
-	unsigned int		disable_jb:1;
-	unsigned int		debug:1;
+	unsigned int		call_flags;
 };
 
 
-
+/**
+ * The main entry point into call objects for signalling events is the call-ID:
+ * Therefore the main entry point is the global hash table rtpe_callhash (protected by rtpe_callhash_lock),
+ * which uses call-IDs as keys and call objects as values,
+ * while holding a reference to each contained call.
+ */
 extern rwlock_t rtpe_callhash_lock;
 extern GHashTable *rtpe_callhash;
 extern struct call_iterator_list rtpe_call_iterators[NUM_CALL_ITERATORS];
 
-extern struct global_stats_gauge rtpe_stats_gauge;
-extern struct global_stats_gauge_min_max rtpe_stats_gauge_graphite_min_max;
-extern struct global_stats_gauge_min_max rtpe_stats_gauge_graphite_min_max_interval;
-
-#define RTPE_GAUGE_SET(field, num) \
-	do { \
-		atomic64_set(&rtpe_stats_gauge.field, num); \
-		RTPE_GAUGE_SET_MIN_MAX(field, rtpe_stats_gauge_graphite_min_max, num); \
-	} while (0)
-#define RTPE_GAUGE_ADD(field, num) \
-	do { \
-		uint64_t __old = atomic64_add(&rtpe_stats_gauge.field, num); \
-		RTPE_GAUGE_SET_MIN_MAX(field, rtpe_stats_gauge_graphite_min_max, __old + num); \
-	} while (0)
-#define RTPE_GAUGE_INC(field) RTPE_GAUGE_ADD(field, 1)
-#define RTPE_GAUGE_DEC(field) RTPE_GAUGE_ADD(field, -1)
-
-extern struct global_stats_ax rtpe_stats;
-extern struct global_stats_counter rtpe_stats_interval;	// accumulators copied out once per interval
-extern struct global_stats_counter rtpe_stats_cumulative;	// total, cumulative
-extern struct global_stats_ax rtpe_stats_graphite;
-extern struct global_stats_counter rtpe_stats_graphite_interval; // copied out when graphite stats run
-extern struct global_stats_min_max rtpe_stats_graphite_min_max; // running min/max
-extern struct global_stats_min_max rtpe_stats_graphite_min_max_interval; // updated once per graphite run
-
-#define RTPE_STATS_ADD(field, num) \
-	do { \
-		atomic64_add(&rtpe_stats.ax.field, num); \
-		atomic64_add(&rtpe_stats_cumulative.field, num); \
-		atomic64_add(&rtpe_stats_graphite.ax.field, num); \
-	} while (0)
-#define RTPE_STATS_INC(field) RTPE_STATS_ADD(field, 1)
 
 
 int call_init(void);
 void call_free(void);
 
 struct call_monologue *__monologue_create(struct call *call);
+void __monologue_free(struct call_monologue *m);
 void __monologue_tag(struct call_monologue *ml, const str *tag);
 void __monologue_viabranch(struct call_monologue *ml, const str *viabranch);
 struct packet_stream *__packet_stream_new(struct call *call);
-void __add_subscription(struct call_monologue *ml, struct call_monologue *other, bool offer_answer,
-		unsigned int media_offset, bool rtcp_only, bool egress);
+void __add_subscription(struct call_monologue *ml, struct call_monologue *other,
+		unsigned int media_offset, const struct sink_attrs *);
+struct call_subscription *call_get_call_subscription(GHashTable *ht, struct call_monologue *ml);
 void free_sink_handler(void *);
-void __add_sink_handler(GQueue *, struct packet_stream *, bool rtcp_only, bool transcoding);
+void __add_sink_handler(GQueue *, struct packet_stream *, const struct sink_attrs *);
 
 void call_subscription_free(void *);
 void call_subscriptions_clear(GQueue *q);
 
 
-struct call *call_get_or_create(const str *callid, bool foreign, bool exclusive);
+struct call *call_get_or_create(const str *callid, bool exclusive);
 struct call *call_get_opmode(const str *callid, enum call_opmode opmode);
 void call_make_own_foreign(struct call *c, bool foreign);
-int call_get_mono_dialogue(struct call_monologue *dialogue[2], struct call *call, const str *fromtag,
+int call_get_mono_dialogue(struct call_subscription *dialogue[2], struct call *call, const str *fromtag,
 		const str *totag,
 		const str *viabranch);
 struct call_monologue *call_get_monologue(struct call *call, const str *fromtag);
 struct call_monologue *call_get_or_create_monologue(struct call *call, const str *fromtag);
 struct call *call_get(const str *callid);
-int monologue_offer_answer(struct call_monologue *dialogue[2], GQueue *streams, struct sdp_ng_flags *flags);
+int monologue_offer_answer(struct call_subscription *dialogue[2], GQueue *streams, struct sdp_ng_flags *flags);
+__attribute__((nonnull(1, 2, 3, 5)))
 void codecs_offer_answer(struct call_media *media, struct call_media *other_media,
-		struct stream_params *sp, struct sdp_ng_flags *flags);
+		struct stream_params *sp, struct sdp_ng_flags *flags, struct call_subscription *dialogue[2]);
 int monologue_publish(struct call_monologue *ml, GQueue *streams, struct sdp_ng_flags *flags);
 int monologue_subscribe_request(const GQueue *srcs, struct call_monologue *dst, struct sdp_ng_flags *);
 int monologue_subscribe_answer(struct call_monologue *dst, struct sdp_ng_flags *,
 		GQueue *);
 int monologue_unsubscribe(struct call_monologue *dst, struct sdp_ng_flags *);
-int monologue_destroy(struct call_monologue *ml);
-int call_delete_branch(const str *callid, const str *branch,
+void monologue_destroy(struct call_monologue *ml);
+int call_delete_branch_by_id(const str *callid, const str *branch,
+	const str *fromtag, const str *totag, bencode_item_t *output, int delete_delay);
+int call_delete_branch(struct call *, const str *branch,
 	const str *fromtag, const str *totag, bencode_item_t *output, int delete_delay);
 void call_destroy(struct call *);
 struct call_media *call_media_new(struct call *call);
 void call_media_free(struct call_media **mdp);
 enum call_stream_state call_stream_state_machine(struct packet_stream *);
 void call_media_state_machine(struct call_media *m);
-void call_media_unkernelize(struct call_media *media);
-void dialogue_unkernelize(struct call_monologue *ml);
-void __monologue_unkernelize(struct call_monologue *monologue);
+void call_media_unkernelize(struct call_media *media, const char *);
+void dialogue_unkernelize(struct call_monologue *ml, const char *);
+void __monologue_unkernelize(struct call_monologue *monologue, const char *);
+void update_init_subscribers(struct call_monologue *ml, enum call_opmode opmode);
 
 int call_stream_address46(char *o, struct packet_stream *ps, enum stream_address_format format,
 		int *len, const struct local_intf *ifa, bool keep_unspec);
 
 void add_total_calls_duration_in_interval(struct timeval *interval_tv);
-void call_timer(void *ptr);
+enum thread_looper_action call_timer(void);
 
 void __rtp_stats_update(GHashTable *dst, struct codec_store *);
 int __init_stream(struct packet_stream *ps);
@@ -748,10 +828,10 @@ INLINE str *call_str_init_dup(struct call *c, char *s) {
 	str_init(&t, s);
 	return call_str_dup(c, &t);
 }
-INLINE void __call_unkernelize(struct call *call) {
+INLINE void __call_unkernelize(struct call *call, const char *reason) {
 	for (GList *l = call->monologues.head; l; l = l->next) {
 		struct call_monologue *ml = l->data;
-		__monologue_unkernelize(ml);
+		__monologue_unkernelize(ml, reason);
 	}
 }
 INLINE endpoint_t *packet_stream_local_addr(struct packet_stream *ps) {
@@ -761,7 +841,7 @@ INLINE endpoint_t *packet_stream_local_addr(struct packet_stream *ps) {
 		return &ps->last_local_endpoint;
 	static endpoint_t dummy = {
 		.address = {
-			.u.ipv4.s_addr = 0,
+			.ipv4.s_addr = 0,
 		},
 		.port = 0,
 	};
