@@ -8,8 +8,7 @@
 #include "bencode.h"
 #include "socket.h"
 #include "call.h"
-
-
+#include "sdp.h"
 
 struct call;
 struct call_stats;
@@ -37,11 +36,14 @@ struct sdp_ng_flags {
 	sockfamily_t *address_family;
 	int tos;
 	str record_call_str;
+	str output_dest;
 	str metadata;
 	str label;
 	str set_label;
+	str to_label;
 	str address;
 	sockaddr_t xmlrpc_callback;
+	endpoint_t dtmf_log_dest;
 	GQueue codec_strip;
 	GHashTable *codec_except;
 	GQueue codec_offer;
@@ -52,8 +54,15 @@ struct sdp_ng_flags {
 	GHashTable *codec_set;
 	int ptime,
 	    rev_ptime;
-	GHashTable *sdes_no;
+	GHashTable *sdes_no;		/* individual crypto suites which are excluded */
+	GHashTable *sdes_only;		/* individual crypto suites which are only accepted */
+	GQueue sdes_order;		/* the order, in which crypto suites are being added to the SDP */
+	GQueue sdes_offerer_pref;	/* preferred crypto suites to be selected for the offerer */
 	str dtls_fingerprint;
+
+	/* commands to manipulate attr lines in SDP */
+	struct sdp_manipulations * sdp_manipulations[__MT_MAX];
+
 	enum {
 		ICE_DEFAULT = 0,
 		ICE_REMOVE,
@@ -75,10 +84,17 @@ struct sdp_ng_flags {
 		MEO_BKW,
 		MEO_BOTH,
 	} media_echo:3;
+	enum {
+		ALL_NONE = 0,
+		ALL_ALL,
+		ALL_OFFER_ANSWER,
+		ALL_NON_OFFER_ANSWER,
+		ALL_FLOWS,
+	} all;
 	enum endpoint_learning el_option;
 	enum block_dtmf_mode block_dtmf_mode;
 	int delay_buffer;
-	int frequency;
+	GArray *frequencies;
 	int volume;
 	char digit;
 	str trigger;
@@ -94,6 +110,13 @@ struct sdp_ng_flags {
 	long long db_id;
 	long long duration;
 	long long pause;
+	long long start_pos;
+	enum {
+		AP_DEFAULT = 0,
+		AP_OFF,
+		AP_TRANSCODING,
+		AP_FORCE,
+	} audio_player:2;
 	unsigned int asymmetric:1,
 	             protocol_accept:1,
 	             no_redis_update:1,
@@ -104,6 +127,7 @@ struct sdp_ng_flags {
 	             replace_origin:1,
 	             replace_sess_conn:1,
 	             replace_sdp_version:1,
+	             force_inc_sdp_ver:1,
 	             replace_username:1,
 	             replace_sess_name:1,
 	             replace_zero_address:1,
@@ -125,20 +149,27 @@ struct sdp_ng_flags {
 	             media_handover:1,
 	             dtls_passive:1,
 	             dtls_reverse_passive:1,
-	             osrtp_accept:1,
+	             osrtp_accept_legacy:1,
+	             osrtp_accept_rfc:1,
 	             osrtp_offer:1,
+	             osrtp_offer_legacy:1,
 	             reset:1,
-	             all:1,
 		     egress:1,
 		     siprec:1,
 	             fragment:1,
 	             record_call:1,
+		     discard_recording:1,
+		     exclude_recording:1,
 		     debug:1,
+		     inactive:1,
 	             loop_protect:1,
 	             original_sendrecv:1,
 	             single_codec:1,
 		     reuse_codec:1,
+		     static_codecs:1,
 		     allow_transcoding:1,
+		     allow_asymmetric_codecs:1,
+		     early_media:1,
 		     accept_any:1,
 	             inject_dtmf:1,
 		     detect_dtmf:1,
@@ -163,19 +194,21 @@ struct sdp_ng_flags {
 	             sdes_authenticated_srtp:1,
 	             sdes_lifetime:1,
 	             sdes_pad:1,
-		     sdes_static:1,
+	             sdes_static:1,
+	             sdes_nonew:1,
 	             drop_traffic_start:1,
 	             drop_traffic_stop:1,
 	             passthrough_on:1,
 	             passthrough_off:1,
+		     block_short:1,
 	             disable_jb:1,
 		     nat_wait:1,
 		     pierce_nat:1;
 };
 
 
-extern int trust_address_def;
-extern int dtls_passive_def;
+extern bool trust_address_def;
+extern bool dtls_passive_def;
 
 str *call_request_tcp(char **);
 str *call_lookup_tcp(char **);
@@ -195,6 +228,7 @@ const char *call_query_ng(bencode_item_t *, bencode_item_t *);
 const char *call_list_ng(bencode_item_t *, bencode_item_t *);
 const char *call_start_recording_ng(bencode_item_t *, bencode_item_t *);
 const char *call_stop_recording_ng(bencode_item_t *, bencode_item_t *);
+const char *call_pause_recording_ng(bencode_item_t *, bencode_item_t *);
 const char *call_start_forwarding_ng(bencode_item_t *, bencode_item_t *);
 const char *call_stop_forwarding_ng(bencode_item_t *, bencode_item_t *);
 const char *call_block_dtmf_ng(bencode_item_t *, bencode_item_t *);
@@ -208,9 +242,10 @@ const char *call_stop_media_ng(bencode_item_t *, bencode_item_t *);
 const char *call_play_dtmf_ng(bencode_item_t *, bencode_item_t *);
 void ng_call_stats(struct call *call, const str *fromtag, const str *totag, bencode_item_t *output,
 		struct call_stats *totals);
-const char *call_publish_ng(bencode_item_t *, bencode_item_t *, const char *, const endpoint_t *);
+const char *call_publish_ng(struct ng_buffer *, bencode_item_t *, bencode_item_t *, const char *,
+		const endpoint_t *);
 const char *call_subscribe_request_ng(bencode_item_t *, bencode_item_t *);
-const char *call_subscribe_answer_ng(bencode_item_t *, bencode_item_t *);
+const char *call_subscribe_answer_ng(struct ng_buffer *, bencode_item_t *, bencode_item_t *);
 const char *call_unsubscribe_ng(bencode_item_t *, bencode_item_t *);
 
 void save_last_sdp(struct call_monologue *ml, str *sdp, GQueue *parsed, GQueue *streams);
@@ -222,5 +257,22 @@ int call_interfaces_init(void);
 void call_interfaces_free(void);
 void call_interfaces_timer(void);
 
+INLINE struct sdp_manipulations *sdp_manipulations_get_by_id(struct sdp_ng_flags *f, enum media_type id) {
+	if (id < 0 || id >= G_N_ELEMENTS(f->sdp_manipulations))
+		return NULL;
+	if (!f->sdp_manipulations[id])
+		f->sdp_manipulations[id] = g_slice_alloc0(sizeof(*f->sdp_manipulations[id]));
+	return f->sdp_manipulations[id];
+}
+
+INLINE struct sdp_manipulations *sdp_manipulations_get_by_name(struct sdp_ng_flags *f, const str *s) {
+	if (!str_cmp(s, "none") || !str_cmp(s, "global"))
+		return sdp_manipulations_get_by_id(f, MT_UNKNOWN);
+	enum media_type id = codec_get_type(s);
+	if (id == MT_OTHER)
+		return NULL;
+	return sdp_manipulations_get_by_id(f, id);
+}
+
 
 #endif
