@@ -18,6 +18,7 @@
 
 struct rtpengine_target_info;
 struct rtpengine_destination_info;
+struct rtpengine_send_packet_info;
 struct re_address;
 struct rtpengine_ssrc_stats;
 
@@ -37,15 +38,16 @@ int kernel_setup_table(unsigned int);
 
 int kernel_add_stream(struct rtpengine_target_info *);
 int kernel_add_destination(struct rtpengine_destination_info *);
-int kernel_del_stream(const struct re_address *);
+int kernel_del_stream_stats(struct rtpengine_command_del_target_stats *);
 GList *kernel_list(void);
-int kernel_update_stats(const struct re_address *a, struct rtpengine_stats_info *out);
+int kernel_update_stats(struct rtpengine_command_stats *);
 
 unsigned int kernel_add_call(const char *id);
 int kernel_del_call(unsigned int);
 
 unsigned int kernel_add_intercept_stream(unsigned int call_idx, const char *id);
 
+int kernel_send_rtcp(struct rtpengine_send_packet_info *info, const char *buf, size_t len);
 
 
 
