@@ -4,7 +4,7 @@
 #include "log.h"
 #include "main.h"
 #include "ssrc.h"
-#include "aux.h"
+#include "helpers.h"
 
 int _log_facility_rtcp;
 int _log_facility_cdr;
@@ -14,11 +14,10 @@ struct rtpengine_config initial_rtpe_config;
 struct poller *rtpe_poller;
 struct poller_map *rtpe_poller_map;
 GString *dtmf_logs;
-struct control_ng *rtpe_control_ng[2];
+GQueue rtpe_control_ng = G_QUEUE_INIT;
 
 static str *sdup(char *s) {
-	str r;
-	str_init(&r, s);
+	str r = STR_INIT(s);
 	return str_dup(&r);
 }
 static void queue_dump(GString *s, GQueue *q) {
@@ -41,8 +40,8 @@ static struct call call;
 static struct sdp_ng_flags flags;
 static struct call_media *media_A;
 static struct call_media *media_B;
-struct call_monologue ml_A;
-struct call_monologue ml_B;
+struct call_monologue *ml_A;
+struct call_monologue *ml_B;
 struct stream_params rtp_types_sp;
 
 #define start() __start(__FILE__, __LINE__)
@@ -60,8 +59,7 @@ static void __cleanup(void) {
 	g_queue_clear_full(&flags.codec_accept, free);
 	g_queue_clear_full(&flags.codec_consume, free);
 	g_queue_clear_full(&flags.codec_mask, free);
-	free_ssrc_hash(&ml_A.ssrc_hash);
-	free_ssrc_hash(&ml_B.ssrc_hash);
+	g_queue_clear(&call.monologues);
 
 	codec_store_cleanup(&rtp_types_sp.codecs);
 	memset(&flags, 0, sizeof(flags));
@@ -72,8 +70,6 @@ static void __init(void) {
 	rtp_types_sp.rtp_endpoint.port = 9;
 	flags.codec_except = g_hash_table_new_full(str_case_hash, str_case_equal, free, NULL);
 	flags.codec_set = g_hash_table_new_full(str_case_hash, str_case_equal, free, free);
-	ml_A.ssrc_hash = create_ssrc_hash_call();
-	ml_B.ssrc_hash = create_ssrc_hash_call();
 }
 static struct packet_stream *ps_new(struct call *c) {
 	struct packet_stream *ps = malloc(sizeof(*ps));
@@ -93,17 +89,17 @@ static void __start(const char *file, int line) {
 	call.tags = g_hash_table_new(g_str_hash, g_str_equal);
 	str_init(&call.callid, "test-call");
 	bencode_buffer_init(&call.buffer);
+	ml_A = __monologue_create(&call);
+	ml_B = __monologue_create(&call);
 	media_A = call_media_new(&call); // originator
 	media_B = call_media_new(&call); // output destination
 	g_queue_push_tail(&media_A->streams, ps_new(&call));
 	g_queue_push_tail(&media_B->streams, ps_new(&call));
-	ZERO(ml_A);
-	ZERO(ml_B);
-	str_init(&ml_A.tag, "tag_A");
-	media_A->monologue = &ml_A;
+	str_init(&ml_A->tag, "tag_A");
+	media_A->monologue = ml_A;
 	media_A->protocol = &transport_protocols[PROTO_RTP_AVP];
-	str_init(&ml_B.tag, "tag_B");
-	media_B->monologue = &ml_B;
+	str_init(&ml_B->tag, "tag_B");
+	media_B->monologue = ml_B;
 	media_B->protocol = &transport_protocols[PROTO_RTP_AVP];
 	__init();
 }
@@ -117,17 +113,17 @@ static void __start(const char *file, int line) {
 static void codec_set(char *c) {
 	// from call_ng_flags_str_ht_split
 	c = strdup(c);
-	str s;
-	str_init(&s, c);
+	str s = STR_INIT(c);
 	str splitter = s;
 
 	while (1) {
 		g_hash_table_replace(flags.codec_set, str_dup(&splitter), str_dup(&s));
-		char *c = memrchr(splitter.s, '/', splitter.len);
-		if (!c)
+		char *cp = memrchr(splitter.s, '/', splitter.len);
+		if (!cp)
 			break;
-		splitter.len = c - splitter.s;
+		splitter.len = cp - splitter.s;
 	}
+	free(c);
 }
 #endif
 
@@ -137,11 +133,15 @@ static void codec_set(char *c) {
 //}
 //#define ht_set(ht, s) __ht_set(flags.ht, #s)
 
-#define sdp_pt_fmt(num, codec, clockrate, fmt) \
-	__sdp_pt_fmt(num, (str) STR_CONST_INIT(#codec), clockrate, (str) STR_CONST_INIT(#codec "/" #clockrate), \
-			(str) STR_CONST_INIT(#codec "/" #clockrate "/1"), (str) STR_CONST_INIT(fmt))
+#define sdp_pt_fmt_ch(num, codec, clockrate, channels, fmt) \
+	__sdp_pt_fmt(num, (str) STR_CONST_INIT(#codec), clockrate, channels, (str) STR_CONST_INIT(#codec "/" #clockrate), \
+			(str) STR_CONST_INIT(#codec "/" #clockrate "/" #channels), (str) STR_CONST_INIT(fmt))
 
-static void __sdp_pt_fmt(int num, str codec, int clockrate, str full_codec, str full_full, str fmt) {
+#define sdp_pt_fmt(num, codec, clockrate, fmt) sdp_pt_fmt_ch(num, codec, clockrate, 1, fmt)
+#define sdp_pt_fmt_s(num, codec, clockrate, fmt) sdp_pt_fmt_ch(num, codec, clockrate, 2, fmt)
+
+static void __sdp_pt_fmt(int num, str codec, int clockrate, int channels, str full_codec, str full_full, str fmt) {
+	str *fmtdup = str_dup(&fmt);
 	struct rtp_payload_type pt = (struct rtp_payload_type) {
 		.payload_type = num,
 		.encoding_with_params = full_codec,
@@ -149,8 +149,8 @@ static void __sdp_pt_fmt(int num, str codec, int clockrate, str full_codec, str
 		.encoding = codec,
 		.clock_rate = clockrate,
 		.encoding_parameters = STR_CONST_INIT(""),
-		.channels = 1,
-		.format_parameters = fmt,
+		.channels = channels,
+		.format_parameters = *fmtdup,
 		.codec_opts = STR_NULL,
 		.rtcp_fb = G_QUEUE_INIT,
 		.ptime = 0,
@@ -158,21 +158,27 @@ static void __sdp_pt_fmt(int num, str codec, int clockrate, str full_codec, str
 		.codec_def = NULL,
 	};
 	codec_store_add_raw(&rtp_types_sp.codecs, rtp_payload_type_dup(&pt));
+	free(fmtdup);
 }
 
 #define sdp_pt(num, codec, clockrate) sdp_pt_fmt(num, codec, clockrate, "")
+#define sdp_pt_s(num, codec, clockrate) sdp_pt_fmt_s(num, codec, clockrate, "")
 
 static void offer(void) {
 	printf("offer\n");
 	flags.opmode = OP_OFFER;
-	codecs_offer_answer(media_B, media_A, &rtp_types_sp, &flags);
+	struct call_subscription subs[2];
+	struct call_subscription *subs_p[2] = {&subs[0], &subs[1]};
+	codecs_offer_answer(media_B, media_A, &rtp_types_sp, &flags, subs_p);
 	__init();
 }
 
 static void answer(void) {
 	printf("answer\n");
 	flags.opmode = OP_ANSWER;
-	codecs_offer_answer(media_A, media_B, &rtp_types_sp, &flags);
+	struct call_subscription subs[2];
+	struct call_subscription *subs_p[2] = {&subs[0], &subs[1]};
+	codecs_offer_answer(media_A, media_B, &rtp_types_sp, &flags, subs_p);
 	__init();
 }
 
@@ -194,14 +200,16 @@ static void __expect(const char *file, int line, GQueue *dumper, const char *cod
 }
 
 #ifdef WITH_AMR_TESTS
-#define check_encoder(side, in_pt, out_pt, out_bitrate) \
-	__check_encoder(__FILE__, __LINE__, media_ ## side, in_pt, out_pt, out_bitrate)
+#define check_encoder(side, otherside, in_pt, out_pt, out_bitrate) \
+	__check_encoder(__FILE__, __LINE__, media_ ## side, media_ ## otherside, in_pt, out_pt, out_bitrate)
 
-static void __check_encoder(const char *file, int line, struct call_media *m, int in_pt, int out_pt,
+static void __check_encoder(const char *file, int line, struct call_media *m,
+		struct call_media *out_m, int in_pt, int out_pt,
 		int out_bitrate)
 {
-	struct codec_handler *ch = g_hash_table_lookup(m->codec_handlers, GINT_TO_POINTER(in_pt));
+	struct codec_handler *ch = codec_handler_lookup(m->codec_handlers, in_pt, out_m);
 	printf("running test %s:%i\n", file, line);
+	assert(ch);
 	assert(ch->source_pt.payload_type == in_pt);
 	if (ch->dest_pt.payload_type != out_pt || ch->dest_pt.bitrate != out_bitrate) {
 		printf("test failed: %s:%i\n", file, line);
@@ -240,14 +248,19 @@ static void __packet_seq_ts(const char *file, int line, struct call_media *media
 	str pl_exp = pload_exp;
 
 	// from media_packet_rtp()
+	struct local_intf lif = { };
+	struct stream_fd sfd = {
+		.local_intf = &lif,
+	};
 	struct media_packet mp = {
 		.call = &call,
 		.media = media,
 		.media_out = other_media,
 		.ssrc_in = get_ssrc_ctx(ssrc, media->monologue->ssrc_hash, SSRC_DIR_INPUT, NULL),
+		.sfd = &sfd,
 	};
 	// from __stream_ssrc()
-	if (!MEDIA_ISSET(media, TRANSCODE))
+	if (!ML_ISSET(media->monologue, TRANSCODING))
 		mp.ssrc_in->ssrc_map_out = ntohl(ssrc);
 	mp.ssrc_out = get_ssrc_ctx(mp.ssrc_in->ssrc_map_out, other_media->monologue->ssrc_hash, SSRC_DIR_OUTPUT, NULL);
 	payload_tracker_add(&mp.ssrc_in->tracker, pt_in & 0x7f);
@@ -308,9 +321,9 @@ static void __packet_seq_ts(const char *file, int line, struct call_media *media
 		printf("\n");
 		uint32_t ts = ntohl(rtp->timestamp);
 		uint16_t seq = ntohs(rtp->seq_num);
-		uint32_t ssrc = ntohl(rtp->ssrc);
-		uint32_t ssrc_pt = ssrc;
-		printf("recv RTP SSRC %x seq %u TS %u PT %u\n", (unsigned int) ssrc,
+		uint32_t rtp_ssrc = ntohl(rtp->ssrc);
+		uint32_t ssrc_pt = rtp_ssrc;
+		printf("recv RTP SSRC %x seq %u TS %u PT %u\n", (unsigned int) rtp_ssrc,
 				(unsigned int) seq, (unsigned int) ts, (unsigned int) rtp->m_pt);
 		if (g_hash_table_contains(rtp_ts_ht, GUINT_TO_POINTER(ssrc_pt))) {
 			uint32_t old_ts = GPOINTER_TO_UINT(g_hash_table_lookup(rtp_ts_ht,
@@ -363,6 +376,10 @@ static void end(void) {
 	bencode_buffer_free(&call.buffer);
 	g_hash_table_destroy(call.tags);
 	g_queue_clear(&call.medias);
+	if (ml_A)
+		__monologue_free(ml_A);
+	if (ml_B)
+		__monologue_free(ml_B);
 	__cleanup();
 	printf("\n");
 }
@@ -522,7 +539,7 @@ int main(void) {
 #ifdef WITH_AMR_TESTS
 	{
 		str codec_name = STR_CONST_INIT("AMR-WB");
-		const codec_def_t *def = codec_find(&codec_name, MT_AUDIO);
+		codec_def_t *def = codec_find(&codec_name, MT_AUDIO);
 		assert(def);
 		if (def->support_encoding && def->support_decoding) {
 			// forward AMR-WB
@@ -530,15 +547,11 @@ int main(void) {
 			sdp_pt(0, PCMU, 8000);
 			transcode(AMR-WB);
 			offer();
-			expect(A, "");
 			expect(A, "0/PCMU/8000");
-			expect(B, "0/PCMU/8000 96/AMR-WB/16000/octet-align=1");
-			expect(B, "");
+			expect(B, "0/PCMU/8000 96/AMR-WB/16000/octet-align=1;mode-change-capability=2");
 			sdp_pt_fmt(96, AMR-WB, 16000, "octet-align=1");
 			answer();
 			expect(A, "0/PCMU/8000");
-			expect(A, "0/PCMU/8000");
-			expect(B, "96/AMR-WB/16000/octet-align=1");
 			expect(B, "96/AMR-WB/16000/octet-align=1");
 			packet_seq(A, 0, PCMU_payload, 0, 0, -1, ""); // nothing due to resampling buffer
 			packet_seq_nf(A, 0, PCMU_payload, 160, 1, 96, AMR_WB_payload);
@@ -551,15 +564,11 @@ int main(void) {
 			sdp_pt_fmt(96, AMR-WB, 16000, "octet-align=1");
 			transcode(PCMU);
 			offer();
-			expect(A, "");
 			expect(A, "96/AMR-WB/16000/octet-align=1");
 			expect(B, "96/AMR-WB/16000/octet-align=1 0/PCMU/8000");
-			expect(B, "");
 			sdp_pt(0, PCMU, 8000);
 			answer();
 			expect(A, "96/AMR-WB/16000/octet-align=1");
-			expect(A, "96/AMR-WB/16000/octet-align=1");
-			expect(B, "0/PCMU/8000");
 			expect(B, "0/PCMU/8000");
 			packet_seq(B, 0, PCMU_payload, 0, 0, -1, ""); // nothing due to resampling buffer
 			packet_seq_nf(B, 0, PCMU_payload, 160, 1, 96, AMR_WB_payload);
@@ -572,15 +581,11 @@ int main(void) {
 			sdp_pt(96, AMR-WB, 16000);
 			transcode(PCMU);
 			offer();
-			expect(A, "");
 			expect(A, "96/AMR-WB/16000");
 			expect(B, "96/AMR-WB/16000 0/PCMU/8000");
-			expect(B, "");
 			sdp_pt(0, PCMU, 8000);
 			answer();
 			expect(A, "96/AMR-WB/16000");
-			expect(A, "96/AMR-WB/16000");
-			expect(B, "0/PCMU/8000");
 			expect(B, "0/PCMU/8000");
 			packet_seq(B, 0, PCMU_payload, 0, 0, -1, ""); // nothing due to resampling buffer
 			packet_seq_nf(B, 0, PCMU_payload, 160, 1, 96, AMR_WB_payload_noe);
@@ -592,7 +597,7 @@ int main(void) {
 
 	{
 		str codec_name = STR_CONST_INIT("AMR");
-		const codec_def_t *def = codec_find(&codec_name, MT_AUDIO);
+		codec_def_t *def = codec_find(&codec_name, MT_AUDIO);
 		assert(def);
 		if (def->support_encoding && def->support_decoding) {
 			// default bitrate
@@ -600,18 +605,14 @@ int main(void) {
 			sdp_pt(0, PCMU, 8000);
 			transcode(AMR);
 			offer();
-			expect(A, "");
 			expect(A, "0/PCMU/8000");
-			expect(B, "0/PCMU/8000 96/AMR/8000/octet-align=1");
-			expect(B, "");
+			expect(B, "0/PCMU/8000 96/AMR/8000/octet-align=1;mode-change-capability=2");
 			sdp_pt_fmt(96, AMR, 8000, "octet-align=1");
 			answer();
 			expect(A, "0/PCMU/8000");
-			expect(A, "0/PCMU/8000");
 			expect(B, "96/AMR/8000/octet-align=1");
-			expect(B, "96/AMR/8000/octet-align=1");
-			check_encoder(A, 0, 96, 0); // uses codec default
-			check_encoder(B, 96, 0, 0);
+			check_encoder(A, B, 0, 96, 0); // uses codec default
+			check_encoder(B, A, 96, 0, 0);
 			end();
 
 			// default bitrate reverse
@@ -619,18 +620,14 @@ int main(void) {
 			sdp_pt(96, AMR, 8000);
 			transcode(PCMU);
 			offer();
-			expect(A, "");
 			expect(A, "96/AMR/8000");
 			expect(B, "96/AMR/8000 0/PCMU/8000");
-			expect(B, "");
 			sdp_pt(0, PCMU, 8000);
 			answer();
 			expect(A, "96/AMR/8000");
-			expect(A, "96/AMR/8000");
 			expect(B, "0/PCMU/8000");
-			expect(B, "0/PCMU/8000");
-			check_encoder(A, 96, 0, 0);
-			check_encoder(B, 0, 96, 0); // uses codec default
+			check_encoder(A, B, 96, 0, 0);
+			check_encoder(B, A, 0, 96, 0); // uses codec default
 			end();
 
 			// specify forward bitrate
@@ -638,18 +635,14 @@ int main(void) {
 			sdp_pt(0, PCMU, 8000);
 			transcode(AMR/8000/1/6700);
 			offer();
-			expect(A, "");
 			expect(A, "0/PCMU/8000");
-			expect(B, "0/PCMU/8000 96/AMR/8000/octet-align=1");
-			expect(B, "");
+			expect(B, "0/PCMU/8000 96/AMR/8000/octet-align=1;mode-change-capability=2");
 			sdp_pt_fmt(96, AMR, 8000, "octet-align=1");
 			answer();
 			expect(A, "0/PCMU/8000");
-			expect(A, "0/PCMU/8000");
 			expect(B, "96/AMR/8000/octet-align=1");
-			expect(B, "96/AMR/8000/octet-align=1");
-			check_encoder(A, 0, 96, 6700);
-			check_encoder(B, 96, 0, 0);
+			check_encoder(A, B, 0, 96, 6700);
+			check_encoder(B, A, 96, 0, 0);
 			end();
 
 			// specify non-default forward bitrate
@@ -657,18 +650,14 @@ int main(void) {
 			sdp_pt(0, PCMU, 8000);
 			transcode(AMR/8000/1/7400);
 			offer();
-			expect(A, "");
 			expect(A, "0/PCMU/8000");
-			expect(B, "0/PCMU/8000 96/AMR/8000/octet-align=1");
-			expect(B, "");
+			expect(B, "0/PCMU/8000 96/AMR/8000/octet-align=1;mode-change-capability=2");
 			sdp_pt_fmt(96, AMR, 8000, "octet-align=1");
 			answer();
 			expect(A, "0/PCMU/8000");
-			expect(A, "0/PCMU/8000");
 			expect(B, "96/AMR/8000/octet-align=1");
-			expect(B, "96/AMR/8000/octet-align=1");
-			check_encoder(A, 0, 96, 7400);
-			check_encoder(B, 96, 0, 0);
+			check_encoder(A, B, 0, 96, 7400);
+			check_encoder(B, A, 96, 0, 0);
 			end();
 
 			// specify reverse bitrate
@@ -677,18 +666,14 @@ int main(void) {
 			transcode(PCMU);
 			codec_set("AMR/8000/1/6700");
 			offer();
-			expect(A, "");
 			expect(A, "96/AMR/8000");
 			expect(B, "96/AMR/8000 0/PCMU/8000");
-			expect(B, "");
 			sdp_pt(0, PCMU, 8000);
 			answer();
 			expect(A, "96/AMR/8000");
-			expect(A, "96/AMR/8000");
 			expect(B, "0/PCMU/8000");
-			expect(B, "0/PCMU/8000");
-			check_encoder(A, 96, 0, 0);
-			check_encoder(B, 0, 96, 6700);
+			check_encoder(A, B, 96, 0, 0);
+			check_encoder(B, A, 0, 96, 6700);
 			end();
 
 			// specify non-default reverse bitrate
@@ -697,18 +682,14 @@ int main(void) {
 			transcode(PCMU);
 			codec_set("AMR/8000/1/7400");
 			offer();
-			expect(A, "");
 			expect(A, "96/AMR/8000");
 			expect(B, "96/AMR/8000 0/PCMU/8000");
-			expect(B, "");
 			sdp_pt(0, PCMU, 8000);
 			answer();
 			expect(A, "96/AMR/8000");
-			expect(A, "96/AMR/8000");
 			expect(B, "0/PCMU/8000");
-			expect(B, "0/PCMU/8000");
-			check_encoder(A, 96, 0, 0);
-			check_encoder(B, 0, 96, 7400);
+			check_encoder(A, B, 96, 0, 0);
+			check_encoder(B, A, 0, 96, 7400);
 			end();
 		}
 	}
@@ -978,17 +959,17 @@ int main(void) {
 	packet_seq_exp(A, 8, PCMA_payload, 1000960, 212, 8, PCMA_payload, 5); // DTMF packets appear lost
 	packet_seq(A, 8, PCMA_payload, 1001120, 213, 8, PCMA_payload);
 	// media blocking
-	ml_A.block_media = 1;
+	ML_SET(ml_A, BLOCK_MEDIA);
 	packet_seq_exp(A, 8, PCMA_payload, 1001280, 214, -1, "", 0);
 	packet_seq_exp(A, 8, PCMA_payload, 1001440, 215, -1, "", 0);
-	ml_A.block_media = 0;
+	ML_CLEAR(ml_A, BLOCK_MEDIA);
 	packet_seq_exp(A, 8, PCMA_payload, 1001600, 216, 8, PCMA_payload, 3); // media packets appear lost
-	call.block_media = 1;
+	CALL_SET(&call, BLOCK_MEDIA);
 	packet_seq_exp(A, 8, PCMA_payload, 1001760, 217, -1, "", 0);
 	packet_seq_exp(A, 8, PCMA_payload, 1001920, 218, -1, "", 0);
-	call.block_media = 0;
+	CALL_CLEAR(&call, BLOCK_MEDIA);
 	packet_seq_exp(A, 8, PCMA_payload, 1002080, 219, 8, PCMA_payload, 3); // media packets appear lost
-	ml_B.block_media = 1;
+	ML_SET(ml_B, BLOCK_MEDIA);
 	packet_seq(A, 8, PCMA_payload, 1002240, 220, 8, PCMA_payload);
 	end();
 
@@ -1043,17 +1024,17 @@ int main(void) {
 	packet_seq_exp(A, 8, PCMA_payload, 1000960, 212, 0, PCMU_payload, 5); // DTMF packets appear lost
 	packet_seq(A, 8, PCMA_payload, 1001120, 213, 0, PCMU_payload);
 	// media blocking
-	ml_A.block_media = 1;
+	ML_SET(ml_A, BLOCK_MEDIA);
 	packet_seq_exp(A, 8, PCMA_payload, 1001280, 214, -1, "", 0);
 	packet_seq_exp(A, 8, PCMA_payload, 1001440, 215, -1, "", 0);
-	ml_A.block_media = 0;
+	ML_CLEAR(ml_A, BLOCK_MEDIA);
 	packet_seq_exp(A, 8, PCMA_payload, 1001600, 214, 0, PCMU_payload, 1); // cheat with the seq here - 216 would get held by the jitter buffer
-	call.block_media = 1;
+	CALL_SET(&call, BLOCK_MEDIA);
 	packet_seq_exp(A, 8, PCMA_payload, 1001760, 215, -1, "", 0);
 	packet_seq_exp(A, 8, PCMA_payload, 1001920, 216, -1, "", 0);
-	call.block_media = 0;
+	CALL_CLEAR(&call, BLOCK_MEDIA);
 	packet_seq_exp(A, 8, PCMA_payload, 1002080, 215, 0, PCMU_payload, 1);
-	ml_B.block_media = 1;
+	ML_SET(ml_B, BLOCK_MEDIA);
 	packet_seq_exp(A, 8, PCMA_payload, 1002240, 216, 0, PCMU_payload, 1);
 	end();
 
@@ -1107,17 +1088,17 @@ int main(void) {
 	packet_seq_exp(A, 0, PCMU_payload, 1000960, 212, 0, PCMU_payload, 5); // DTMF packets appear lost
 	packet_seq(A, 0, PCMU_payload, 1001120, 213, 0, PCMU_payload);
 	// media blocking
-	ml_A.block_media = 1;
+	ML_SET(ml_A, BLOCK_MEDIA);
 	packet_seq_exp(A, 0, PCMU_payload, 1001280, 214, -1, "", 0);
 	packet_seq_exp(A, 0, PCMU_payload, 1001440, 215, -1, "", 0);
-	ml_A.block_media = 0;
+	ML_CLEAR(ml_A, BLOCK_MEDIA);
 	packet_seq_exp(A, 0, PCMU_payload, 1001600, 216, 0, PCMU_payload, 3); // media packets appear lost
-	call.block_media = 1;
+	CALL_SET(&call, BLOCK_MEDIA);
 	packet_seq_exp(A, 0, PCMU_payload, 1001760, 217, -1, "", 0);
 	packet_seq_exp(A, 0, PCMU_payload, 1001920, 218, -1, "", 0);
-	call.block_media = 0;
+	CALL_CLEAR(&call, BLOCK_MEDIA);
 	packet_seq_exp(A, 0, PCMU_payload, 1002080, 219, 0, PCMU_payload, 3); // media packets appear lost
-	ml_B.block_media = 1;
+	ML_SET(ml_B, BLOCK_MEDIA);
 	packet_seq(A, 0, PCMU_payload, 1002240, 220, 0, PCMU_payload);
 	end();
 
@@ -1444,7 +1425,7 @@ int main(void) {
 	transcode(PCMA);
 	transcode(telephone-event);
 	offer();
-	expect(B, "96/opus/48000 8/PCMA/8000 97/telephone-event/48000/0-15 101/telephone-event/8000");
+	expect(B, "96/opus/48000/useinbandfec=1 8/PCMA/8000 97/telephone-event/48000/0-15 101/telephone-event/8000");
 	sdp_pt(96, opus, 48000);
 	sdp_pt(97, telephone-event, 48000);
 	flags.single_codec = 1;
@@ -1606,32 +1587,32 @@ int main(void) {
 	packet_seq(B, 8, PCMA_payload, 0, 0, 8, PCMA_payload);
 	packet_seq(A, 8, PCMA_payload, 160, 1, 8, PCMA_payload);
 	packet_seq(B, 8, PCMA_payload, 160, 1, 8, PCMA_payload);
-	call.silence_media = 1;
+	CALL_SET(&call, SILENCE_MEDIA);
 	packet_seq(A, 8, PCMA_payload, 320, 2, 8, PCMA_silence);
 	packet_seq(B, 8, PCMA_payload, 320, 2, 8, PCMA_silence);
 	packet_seq(A, 8, PCMA_payload, 480, 3, 8, PCMA_silence);
 	packet_seq(B, 8, PCMA_payload, 480, 3, 8, PCMA_silence);
-	call.silence_media = 0;
+	CALL_CLEAR(&call, SILENCE_MEDIA);
 	packet_seq(A, 8, PCMA_payload, 640, 4, 8, PCMA_payload);
 	packet_seq(B, 8, PCMA_payload, 640, 4, 8, PCMA_payload);
 	packet_seq(A, 8, PCMA_payload, 800, 5, 8, PCMA_payload);
 	packet_seq(B, 8, PCMA_payload, 800, 5, 8, PCMA_payload);
-	ml_A.silence_media = 1;
+	ML_SET(ml_A, SILENCE_MEDIA);
 	packet_seq(A, 8, PCMA_payload, 960, 6, 8, PCMA_silence);
 	packet_seq(B, 8, PCMA_payload, 960, 6, 8, PCMA_payload);
 	packet_seq(A, 8, PCMA_payload, 1120, 7, 8, PCMA_silence);
 	packet_seq(B, 8, PCMA_payload, 1120, 7, 8, PCMA_payload);
-	ml_A.silence_media = 0;
+	ML_CLEAR(ml_A, SILENCE_MEDIA);
 	packet_seq(A, 8, PCMA_payload, 1280, 8, 8, PCMA_payload);
 	packet_seq(B, 8, PCMA_payload, 1280, 8, 8, PCMA_payload);
 	packet_seq(A, 8, PCMA_payload, 1440, 9, 8, PCMA_payload);
 	packet_seq(B, 8, PCMA_payload, 1440, 9, 8, PCMA_payload);
-	ml_B.silence_media = 1;
+	ML_SET(ml_B, SILENCE_MEDIA);
 	packet_seq(A, 8, PCMA_payload, 1600, 10, 8, PCMA_payload);
 	packet_seq(B, 8, PCMA_payload, 1600, 10, 8, PCMA_silence);
 	packet_seq(A, 8, PCMA_payload, 1760, 11, 8, PCMA_payload);
 	packet_seq(B, 8, PCMA_payload, 1760, 11, 8, PCMA_silence);
-	ml_B.silence_media = 0;
+	ML_CLEAR(ml_B, SILENCE_MEDIA);
 	packet_seq(A, 8, PCMA_payload, 1920, 12, 8, PCMA_payload);
 	packet_seq(B, 8, PCMA_payload, 1920, 12, 8, PCMA_payload);
 	packet_seq(A, 8, PCMA_payload, 2080, 13, 8, PCMA_payload);
@@ -1652,38 +1633,90 @@ int main(void) {
 	packet_seq(B, 0, PCMU_payload, 0, 0, 0, PCMU_payload);
 	packet_seq(A, 0, PCMU_payload, 160, 1, 0, PCMU_payload);
 	packet_seq(B, 0, PCMU_payload, 160, 1, 0, PCMU_payload);
-	call.silence_media = 1;
+	CALL_SET(&call, SILENCE_MEDIA);
 	packet_seq(A, 0, PCMU_payload, 320, 2, 0, PCMU_silence);
 	packet_seq(B, 0, PCMU_payload, 320, 2, 0, PCMU_silence);
 	packet_seq(A, 0, PCMU_payload, 480, 3, 0, PCMU_silence);
 	packet_seq(B, 0, PCMU_payload, 480, 3, 0, PCMU_silence);
-	call.silence_media = 0;
+	CALL_CLEAR(&call, SILENCE_MEDIA);
 	packet_seq(A, 0, PCMU_payload, 640, 4, 0, PCMU_payload);
 	packet_seq(B, 0, PCMU_payload, 640, 4, 0, PCMU_payload);
 	packet_seq(A, 0, PCMU_payload, 800, 5, 0, PCMU_payload);
 	packet_seq(B, 0, PCMU_payload, 800, 5, 0, PCMU_payload);
-	ml_A.silence_media = 1;
+	ML_SET(ml_A, SILENCE_MEDIA);
 	packet_seq(A, 0, PCMU_payload, 960, 6, 0, PCMU_silence);
 	packet_seq(B, 0, PCMU_payload, 960, 6, 0, PCMU_payload);
 	packet_seq(A, 0, PCMU_payload, 1120, 7, 0, PCMU_silence);
 	packet_seq(B, 0, PCMU_payload, 1120, 7, 0, PCMU_payload);
-	ml_A.silence_media = 0;
+	ML_CLEAR(ml_A, SILENCE_MEDIA);
 	packet_seq(A, 0, PCMU_payload, 1280, 8, 0, PCMU_payload);
 	packet_seq(B, 0, PCMU_payload, 1280, 8, 0, PCMU_payload);
 	packet_seq(A, 0, PCMU_payload, 1440, 9, 0, PCMU_payload);
 	packet_seq(B, 0, PCMU_payload, 1440, 9, 0, PCMU_payload);
-	ml_B.silence_media = 1;
+	ML_SET(ml_B, SILENCE_MEDIA);
 	packet_seq(A, 0, PCMU_payload, 1600, 10, 0, PCMU_payload);
 	packet_seq(B, 0, PCMU_payload, 1600, 10, 0, PCMU_silence);
 	packet_seq(A, 0, PCMU_payload, 1760, 11, 0, PCMU_payload);
 	packet_seq(B, 0, PCMU_payload, 1760, 11, 0, PCMU_silence);
-	ml_B.silence_media = 0;
+	ML_CLEAR(ml_B, SILENCE_MEDIA);
 	packet_seq(A, 0, PCMU_payload, 1920, 12, 0, PCMU_payload);
 	packet_seq(B, 0, PCMU_payload, 1920, 12, 0, PCMU_payload);
 	packet_seq(A, 0, PCMU_payload, 2080, 13, 0, PCMU_payload);
 	packet_seq(B, 0, PCMU_payload, 2080, 13, 0, PCMU_payload);
 	end();
 
+	start();
+	sdp_pt_s(96, opus, 48000);
+	sdp_pt(8, PCMA, 8000);
+	c_accept(opus/48000/2);
+	offer();
+	expect(A, "96/opus/48000/2 8/PCMA/8000");
+	expect(B, "96/opus/48000/2 8/PCMA/8000");
+	sdp_pt(8, PCMA, 8000);
+	answer();
+	expect(A, "96/opus/48000/2");
+	expect(B, "8/PCMA/8000");
+	end();
+
+	start();
+	sdp_pt(96, opus, 48000);
+	sdp_pt(8, PCMA, 8000);
+	c_accept(opus/48000);
+	offer();
+	expect(A, "96/opus/48000 8/PCMA/8000");
+	expect(B, "96/opus/48000 8/PCMA/8000");
+	sdp_pt(8, PCMA, 8000);
+	answer();
+	expect(A, "96/opus/48000");
+	expect(B, "8/PCMA/8000");
+	end();
+
+	start();
+	sdp_pt(96, opus, 48000);
+	sdp_pt(8, PCMA, 8000);
+	c_accept(opus);
+	offer();
+	expect(A, "96/opus/48000 8/PCMA/8000");
+	expect(B, "96/opus/48000 8/PCMA/8000");
+	sdp_pt(8, PCMA, 8000);
+	answer();
+	expect(A, "96/opus/48000");
+	expect(B, "8/PCMA/8000");
+	end();
+
+	start();
+	sdp_pt_s(96, opus, 48000);
+	sdp_pt(8, PCMA, 8000);
+	c_accept(opus);
+	offer();
+	expect(A, "96/opus/48000/2 8/PCMA/8000");
+	expect(B, "96/opus/48000/2 8/PCMA/8000");
+	sdp_pt(8, PCMA, 8000);
+	answer();
+	expect(A, "96/opus/48000/2");
+	expect(B, "8/PCMA/8000");
+	end();
+
 	return 0;
 }
 
